/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008-2011 Brian Fernandes <brian@thegoan.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");


const BIBLEPROT_HANDLER_CONTRACTID = "@mozilla.org/network/protocol;1?name=bible";
const BIBLEPROT_HANDLER_CID = Components.ID("{5b44bfa6-a70b-11dc-8314-0800200c9a66}");
const SWORDPROT_HANDLER_CONTRACTID = "@mozilla.org/network/protocol;1?name=sword";
const SWORDPROT_HANDLER_CID = Components.ID("{20e52130-2400-11dd-bd0b-0800200c9a66}");
const STANDARDURL_CONTRACTID = "@mozilla.org/network/standard-url;1";
const STRING_IS_CONTRACTID = "@mozilla.org/io/string-input-stream;1";


const nsISupports        = Components.interfaces.nsISupports;
const nsIObserver        = Components.interfaces.nsIObserver;
const nsIProtocolHandler = Components.interfaces.nsIProtocolHandler;
const nsIStandardURL     = Components.interfaces.nsIStandardURL;
const nsIURI             = Components.interfaces.nsIURI;
const nsIChannel         = Components.interfaces.nsIChannel;
const nsIRequest         = Components.interfaces.nsIRequest;
const nsIInputStream 	 = Components.interfaces.nsIInputStream;
const nsIStringInputStream = Components.interfaces.nsIStringInputStream;

function BibleProtocolHandler() {
	this.wrappedJSObject = this;
	this.windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);	
};

BibleProtocolHandler.prototype.protocolFlags = nsIProtocolHandler.URI_NORELATIVE |
												nsIProtocolHandler.URI_NOAUTH;

BibleProtocolHandler.prototype.classID = BIBLEPROT_HANDLER_CID;
BibleProtocolHandler.prototype.contractID = BIBLEPROT_HANDLER_CONTRACTID;

if ("URI_LOADABLE_BY_ANYONE" in nsIProtocolHandler) {
  BibleProtocolHandler.prototype.protocolFlags |=
      nsIProtocolHandler.URI_LOADABLE_BY_ANYONE;
}

BibleProtocolHandler.prototype.allowPort = function (port, schem) {
	return false;
};

BibleProtocolHandler.prototype.newURI = function(spec, charset, baseURI) {
	var cls = Components.classes[STANDARDURL_CONTRACTID];
   	var url = cls.createInstance(nsIStandardURL);
   	url.init(nsIStandardURL.URLTYPE_STANDARD, 80, spec, charset, baseURI);
   	return url.QueryInterface(nsIURI);
};

BibleProtocolHandler.prototype.newChannel = function(URI) {
	if (this.jsword == null || !this.jsword.utilsInitialized) {
		var win = this.windowManager.getMostRecentWindow("navigator:browser");
		win.fireBible.initFromProtocol();
	}
	return new BibleChannel(URI, this.jsword, this.windowManager);		
};

BibleProtocolHandler.prototype.getJSword = function() {
	return this.jsword;
}

BibleProtocolHandler.prototype.setJSword = function(jsword) {
	Components.utils.import("resource://firebible/modules/biblechannel.js");
	this.jsword = jsword;
}

BibleProtocolHandler.prototype.getLocation = function() {
	return __LOCATION__;
}

BibleProtocolHandler.prototype.QueryInterface = function(aIID) {
	if (!aIID.equals(nsIProtocolHandler) &&
        !aIID.equals(nsISupports) &&
        !aIID.equals(nsIObserver))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;

};

BibleProtocolHandler.prototype.observe = function(aSubject, aTopic, aData) {
	//do nothing
};

function SwordProtocolHandler() {	
}
SwordProtocolHandler.prototype = new BibleProtocolHandler;
SwordProtocolHandler.prototype.classID = SWORDPROT_HANDLER_CID;
SwordProtocolHandler.prototype.contractID = SWORDPROT_HANDLER_CONTRACTID;

if (XPCOMUtils.generateNSGetFactory) {
    var NSGetFactory = XPCOMUtils.generateNSGetFactory([BibleProtocolHandler, SwordProtocolHandler]);
} else {
    var NSGetModule = XPCOMUtils.generateNSGetModule([BibleProtocolHandler, SwordProtocolHandler]);
}
