/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008-2015 Brian Fernandes <brian@thegoan.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
/*TODO List
*Fix rendering of notes to not be clamped together
*Look at issue where visiting an anchor is interpreted as a reference to "revelations"
*Concord has genbook text with bible references and verses, linking doesn't work
*/

FireBible.prototype.BOOKS_RELOADED = "BOOKS_RELOADED";

FireBible.prototype.init = function() {
	this.initInterfaces();
	this.log("Firebible Version 2.0b3");
	this.name = "chrome://firebible";
	this.booksNeedReloading = false;
	this.bControls = new Array();
	this.observerService.addObserver(this, this.name, false);
	this.verseEx = new RegExp("^(\\d*\\D+)((?:[^a-eg-z\\s]|\\s(?!f))*)", "i");
};

FireBible.prototype.initInterfaces = function() {
	const
	Ci = Components.interfaces;
	const
	Cc = Components.classes;
	this.windowManager = Cc['@mozilla.org/appshell/window-mediator;1']
			.getService(Ci.nsIWindowMediator);
	this.promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"]
			.getService(Ci.nsIPromptService);
	this.observerService = Cc["@mozilla.org/observer-service;1"]
			.getService(Ci.nsIObserverService);
	this._logService = Cc['@mozilla.org/consoleservice;1'].getService();
	this.annotSvc = Cc["@mozilla.org/browser/annotation-service;1"]
			.getService(Ci.nsIAnnotationService);
	this.pHandler = Cc["@mozilla.org/network/protocol;1?name=bible"]
			.getService().wrappedJSObject;
	this.pHandler2 = Cc["@mozilla.org/network/protocol;1?name=sword"]
			.getService().wrappedJSObject;

};

FireBible.prototype.getJSword = function() {
	this.jsword = this.pHandler.getJSword();
	if (!this.jsword) {
		this.jsword = new JSword();
		this.pHandler.setJSword(this.jsword);
		this.pHandler2.setJSword(this.jsword);
		this.jsword.setComponentLocation(this.pHandler.getLocation());
	}
	return this.jsword;
};

FireBible.prototype.initializeMenus = function() {
	var prefs = this.getJSword().fbPrefs;

	this.initMenuItem(prefs, prefs.VERSE_SEPARATE_LINE);
	this.initMenuItem(prefs, prefs.VERSE_SUPERSCRIPT);
	this.initMenuItem(prefs, prefs.SHOW_HEADINGS);
	this.initMenuItem(prefs, prefs.SHOW_NOTES);
	this.initMenuItem(prefs, prefs.SHOW_STRONGS);
	this.initMenuItem(prefs, prefs.SHOW_MORPH);

	var verseFormat = prefs.getIntPref(prefs.VERSE_FORMAT);
	var menuItem = document.getElementById(prefs.VERSE_FORMAT + "."
			+ verseFormat);
	menuItem.setAttribute("checked", true);
};

FireBible.prototype.initMenuItem = function(prefs, id) {
	var menuItem = document.getElementById(id);
	var checked = prefs.getBoolPref(id);
	menuItem.setAttribute("checked", checked);
};

FireBible.prototype.toggleMenuItem = function(menuItem) {
	var checked = menuItem.getAttribute("checked");
	checked = (!checked || checked == "false") ? false : true;
	this.jsword.fbPrefs.setBoolPref(menuItem.id, checked);
	this.reload();
};

FireBible.prototype.setVerseFormat = function(format) {
	this.jsword.fbPrefs.setIntPref(this.jsword.fbPrefs.VERSE_FORMAT, format);
	this.reload();
};

FireBible.prototype.reload = function() {
	if (gBrowser.selectedBrowser.currentURI != null) {
		var uri = gBrowser.selectedBrowser.currentURI.spec;
		if (uri.indexOf("bible://") == 0 || uri.indexOf("sword://") == 0) {
			gBrowser.selectedBrowser.reload();
		}
	}
};

FireBible.prototype.initFromProtocol = function(silent) {
	this.bibleControl.initBibles(silent);
};

FireBible.prototype.initBibleList = function(forceReset, doNotWarn) {
	this.log("initbiblelist " + forceReset + ", dnw: " + doNotWarn);
	var bibles = this.getJSword().getBibleList(forceReset, doNotWarn);
	this.bibleControl.bibles = bibles;
	return (bibles.length > 0);
};

FireBible.prototype.initBookList = function(bible) {
	var firstBible = bible.split(".")[0]; // bible could be a list of period
											// delimited bibles
	this.bibleControl.books = this.getJSword().getBookList(firstBible);
};

FireBible.prototype.onLoad = function() {
	this.strings = document.getElementById("firebible-strings");
	this.manageBookmarks();
	document.addEventListener("FireBibleEvent", this.dictListener, true, true);
	try {
		this.setupUbiquity();
	} catch (e) {
		this
				.log("Error installing Ubiquity support. Please ensure you have Ubiquity 0.6 or higher.");
	}
	this.bibleControl.lastURL = "";
	gBrowser.mTabContainer.addEventListener("select", fireBible.tabSelected,
			true);
	// gBrowser.mTabContainer.addEventListener("load", fireBible.tabLoaded2,
	// true);
	gBrowser.addTabsProgressListener(fireBible.tabLoaded);
};

FireBible.prototype.tabLoaded = {};
FireBible.prototype.tabLoaded.onLocationChange = function(aBrowser,
		aWebProgress, aRequest, aLocation, aFlags) {
	if (aLocation.scheme == "bible" || aLocation.scheme == "sword") {
		if (!fireBible.getJSword().utilsInitialized) {
			return;
		}

		// synch combos
		var url = aLocation.host + aLocation.path;
		if (url != fireBible.bibleControl.lastURL) {
			var temp = fireBible.parseURL(url);
			var bibleParts = temp[0];
			var path = temp[1];
			var modArray = fireBible.getJSword().getModuleArray(bibleParts[0]);
			var mod = modArray.config;
			var type = modArray.bCat;
			if (type == "bible") {
				var bChapter = fireBible.getJSword().libSword.parseVerseKey(
						mod, path.split("#")[0]);
				for (var i = 0; i < bibleParts.length; i++) {
					bibleParts[i] = fireBible.getJSword().getModule(
							bibleParts[i]);
				}
				var key = bChapter.split(",");
				fireBible.bibleControl.syncCombos(bibleParts, key[0], key[1]);
				// fireBible.bibleControl.lastURL = url;
			}
			
			if (aWebProgress.isLoadingDocument) {
				// set the tab Icon
				var aTab = null;
				var mTabs = gBrowser.mTabContainer.childNodes;
				for (var i = 0; i < mTabs.length; i++) {
					if (mTabs[i].linkedBrowser == aBrowser) {
						aTab = mTabs[i];
						break;
					}
				}
				var icon;
				fireBible.log("class is " + modArray.bClass + ", " + modArray.bCat);
				if (modArray.bClass == "commentary") {
					icon = "resource://firebible/skin/book-c16.png";
				} else if (modArray.bClass == "genbook") {
					icon = "resource://firebible/skin/book16.png";
				} else if (modArray.bClass == "dailydevotional") {
					icon = "resource://firebible/skin/book-r16.png";
				} else if (modArray.bClass == "maps") {
					icon = "resource://firebible/skin/book-m16.png";
				} else {
					icon = "resource://firebible/skin/bible16.png";
				}
				gBrowser.setIcon(aTab, icon);
			}
			
			//load commentary?
			fireBible.lastBibleURI = path;			
			var sidebarWindow = window.document.getElementById("sidebar").contentWindow;
			if (sidebarWindow && sidebarWindow.location.href == "chrome://firebible/content/sidebar.xul" && sidebarWindow.fbItemList) {
				sidebarWindow.fbItemList.loadCommentary();
			}
		}
		
		
	}
};

FireBible.prototype.parseURL = function(url) {
	var parts = url.split("/");
	var bibleParts = new Array();
	var path;
	if (parts.length >= 2 && parts[1].length > 0 && parts[1][0] != "#") {
		bibleParts = parts[0].split(".");
		parts.shift(); //remove first element
		path = parts.join("/"); //join additional, unnecessary splits if present
	} else {
		bibleParts[0] = fireBible.getJSword().getDefaultBible();
		path = parts[0];
	}
	return [bibleParts, path];
};

FireBible.prototype.tabLoaded2 = function(event) {
	var url = gBrowser.mCurrentBrowser.contentDocument.URL;
	if (url && (url.indexOf("bible://") == 0 || url.indexOf("sword://") == 0)) {
		alert('loaded');
	}
};

FireBible.prototype.tabSelected = function(event) {
	var url = gBrowser.mCurrentBrowser.contentDocument.URL;
	if (url && (url.indexOf("bible://") == 0 || url.indexOf("sword://") == 0)) {
		var aWebProgress = {};
		aWebProgress.isLoadingDocument = false;
		var aLocation = {};
		aLocation.scheme = "bible";
		aLocation.host = url.substring(8);
		aLocation.path = "";
		fireBible.tabLoaded.onLocationChange(null, aWebProgress, null,
				aLocation, null);
	}
	// fireBible.tabLoaded.onLocationChange(null, aWebProgress, null, null,
	// null);
	// if (url && (url.indexOf("bible://") == 0 || url.indexOf("sword://") ==
	// 0)) {
	// if (url != fireBible.bibleControl.lastURL) {
	// FIXME: synch combos
	// var combo =
	// fireBible.getJSword().utilKlass.extractCombos(unescape(url.substring(8)));
	// fireBible.bibleControl.syncCombos(combo);
	// fireBible.bibleControl.lastURL = url;
	// }
	// }
};

FireBible.prototype.loadDictItem = function() {
	if (!this.dictKey)
		return;
	var sidebarWindow = document.getElementById("sidebar").contentWindow;
	sidebarWindow.fbItemList.loadDictionary(this.dictKey);
};

FireBible.prototype.dictListener = function(e) {
	fireBible.dictKey = e.detail.key;
	/*var type = e.target.getAttribute("type");
	if (type && type == "2") {
		fireBible.dictKey = "M" + fireBible.dictKey;
	}*/
	var sidebarWindow = document.getElementById("sidebar").contentWindow;
	if (sidebarWindow.location.href != "chrome://firebible/content/sidebar.xul") {
		window.toggleSidebar('viewFireBibleSidebar');
	} else {
		fireBible.loadDictItem();
	}
};

FireBible.prototype.setupUbiquity = function() {
	var url = this.getJSword().getFileURI(
			this.pHandler.getLocation().parent.parent.path
					+ "/content/ubiquity.js");
	// link to the ubiquity setup module.
	var jsm = {};
	Components.utils.import("resource://ubiquity/modules/setup.js", jsm);
	if (jsm.UbiquitySetup) {
		jsm.UbiquitySetup.preload(function() {
			try {
				jsm.UbiquitySetup.createServices().feedManager
						.addSubscribedFeed({
							url : url,
							sourceUrl : url,
							canAutoUpdate : true,
							isBuiltIn : false
						});
			} catch (e) {
				this.log("Error installing Ubiquity sword command: " + e);
			}
		});
	}
};

FireBible.prototype.getChildFolder = function(bmsvc, aFolder, aSubFolder) {
	// see: http://forums.mozillazine.org/viewtopic.php?f=19&t=2060171
	if (bmsvc.getChildFolder) { // avaialable in FF 3.0 - 3.6 only
		return bmsvc.getChildFolder(aFolder, aSubFolder);
	}
	var htService = Components.classes["@mozilla.org/browser/nav-history-service;1"]
			.getService(Components.interfaces.nsINavHistoryService);
	var query = htService.getNewQuery();
	var options = htService.getNewQueryOptions();
	query.setFolders([ aFolder ], 1);
	options.queryType = options.QUERY_TYPE_BOOKMARKS;
	var result = htService.executeQuery(query, options);
	var rootNode = result.root;
	rootNode.containerOpen = true;
	for (var i = 0; i < rootNode.childCount; i++) {
		var node = rootNode.getChild(i);
		if (node.type == node.RESULT_TYPE_FOLDER && node.title == aSubFolder) {
			return node.itemId;

		}
	}
	return 0;
};

FireBible.prototype.manageBookmarks = function() {
	var bmsvc = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"];
	if (!bmsvc) return;
	bmsvc = bmsvc.getService(Components.interfaces.nsINavBookmarksService);

	var ios = Components.classes["@mozilla.org/network/io-service;1"]
			.getService(Components.interfaces.nsIIOService);
	var tFolder = bmsvc.bookmarksMenuFolder;
	var history = this.getChildFolder(bmsvc, tFolder, this._("history.folder"));
	var bookmarks = this.getChildFolder(bmsvc, tFolder, this._("bookmarks.folder"));
	var index;
	if (history) {
		index = bmsvc.getItemIndex(history) + 1;
	} else if (bookmarks) {
		index = bmsvc.getItemIndex(bookmarks);
	} else {
		index = bmsvc.DEFAULT_INDEX;
	}

	if (!history) {
		// create history queries
		history = bmsvc.createFolder(tFolder, this._("history.folder"), index);
		this.insertBookmark(bmsvc, ios,
				"sort=4&maxResults=40&!annotation=firebible/bookCategory/dictionary",
				this._("lastForty.folder"), history);
		this.insertBookmark(bmsvc, ios,
				"beginTimeRef=2&beginTime=-7200000000&endTimeRef=2&endTime=0&sort=4",
				this._("last2hrs.folder"), history);
		this.insertBookmark(bmsvc, ios, 
				"sort=8&maxResults=20", 
				this._("mostVisited.folder"), history);
		bmsvc.insertSeparator(history, bmsvc.DEFAULT_INDEX);
		
		
		this.insertBookmark(bmsvc, ios,
				"sort=4&maxResults=40&annotation=firebible/bookCategory/bible",
				this._("biblesComm.folder"), history);
		this.insertBookmark(bmsvc, ios,
				"sort=4&maxResults=40&annotation=firebible/bookCategory/genbook",
				this._("genBooks.folder"), history);
		this.insertBookmark(bmsvc,ios,
				"sort=4&maxResults=40&annotation=firebible/bookCategory/dictionary",
				this._("dict.folder"), history);		
		bmsvc.insertSeparator(history, bmsvc.DEFAULT_INDEX);
		
		this.insertBookmark(bmsvc, ios, 
				"beginTimeRef=1&beginTime=0&sort=1",
				this._("today.folder"), history);
		this.insertBookmark(bmsvc, ios,
				"beginTimeRef=1&beginTime=-86400000000&endTimeRef=1&endTime=0&sort=1",
				this._("yest.folder"), history);
		this.insertBookmark(bmsvc, ios,
				"beginTimeRef=1&beginTime=-172800000000&endTimeRef=1&endTime=-86400000000&sort=1",
				this._("twoDaysAgo.folder"), history);
		this.insertBookmark(bmsvc, ios,
				"beginTimeRef=1&beginTime=-259200000000&endTimeRef=1&endTime=-172800000000&sort=1",
				this._("threeDaysAgo.folder"), history);
		this.insertBookmark(bmsvc, ios,
				"beginTimeRef=1&beginTime=-345600000000&endTimeRef=1&endTime=-259200000000&sort=1",
				this._("fourDaysAgo.folder"), history);
		this.insertBookmark(bmsvc, ios,
				"beginTimeRef=1&beginTime=-432000000000&endTimeRef=1&endTime=-345600000000&sort=1",
				this._("fiveDaysAgo.folder"), history);
		this.insertBookmark(bmsvc, ios,
				"beginTimeRef=1&beginTime=-518400000000&endTimeRef=1&endTime=-432000000000&sort=1",
				this._("sixDaysAgo.folder"), history);
		this.insertBookmark(bmsvc, ios,
				"&endTimeRef=1&endTime=-518400000000&sort=4&maxResults=40",
				this._("olderThanSixDays.folder"), history);
	}

	if (!bookmarks) {
		// create bookmark queries
		bookmarks = bmsvc.createFolder(tFolder, this._("bookmarks.folder"), index);
		this.insertBookmark(bmsvc, ios, 
				"queryType=1&sort=1", 
				this._("all.folder"), bookmarks);
		bmsvc.insertSeparator(bookmarks, bmsvc.DEFAULT_INDEX);
		
		this.insertBookmark(bmsvc, ios,
				"annotation=firebible/bookCategory/bible&queryType=1&sort=1",
				this._("biblesComm.folder"), bookmarks);
		this.insertBookmark(bmsvc, ios,
				"annotation=firebible/bookCategory/genbook&queryType=1&sort=1",
				this._("genBooks.folder"), bookmarks);
		this.insertBookmark(bmsvc, ios,
				"annotation=firebible/bookCategory/dictionary&queryType=1&sort=1",
				this._("dict.folder"), bookmarks);
		bmsvc.insertSeparator(bookmarks, bmsvc.DEFAULT_INDEX);
		
		var bQuery = "folder=BOOKMARKS_MENU&folder=UNFILED_BOOKMARKS&folder=TOOLBAR&maxResults=20&queryType=1&";
		this.insertBookmark(bmsvc, ios, 
				bQuery + "sort=12", 
				this._("recentlyBookmarked.folder"), bookmarks);
		this.insertBookmark(bmsvc, ios, 
				bQuery + "sort=4", 
				this._("recentlyVisited.folder"), bookmarks);
		this.insertBookmark(bmsvc, ios, 
				bQuery + "sort=8", 
				this._("mostVisited.folder"), bookmarks);
	}

	this.fbFolderIds = [ history, bookmarks ];
	this.bibleControl.initFBFolder(this.fbFolderIds);
};

FireBible.prototype.insertBookmark = function(bmsvc, ios, place, name, folder) {
//	var uri = ios.newURI("place:terms=bible://&OR&terms=sword://&" + place,
//	var uri = ios.newURI("place:terms=bible://&" + place,
	var uri = ios.newURI("place:tag=firebible&" + place, null, null);
	bmsvc.insertBookmark(folder, uri, bmsvc.DEFAULT_INDEX, name);
};

FireBible.prototype.initUtils = function(silent) {
	var js = this.getJSword();
	var inited = false;
	if (!js.initializing) {
		js.initializing = true;
		try {
			inited = js.initUtils(silent);
		} catch (e) {
			e = js.getRootException(e);
			this.log(e);
			this.printStackTrace(e);
		}
		js.initializing = false;
	} else if (!silent) {
		this.promptService.alert(window, this._("info.title"), this
				._("jswordInitializing.message"));
	}
	return inited;
};

FireBible.prototype.register = function(element) {
	this.bibleControl = element;
	if (this.fbFolderIds) {
		this.bibleControl.initFBFolder(this.fbFolderIds);
	}
};

FireBible.prototype.onUnload = function() {
};

FireBible.prototype.getGBrowser = function() {
	return gBrowser;
};

FireBible.prototype.log = function(message) {
	if (this.jsword
			&& !this.jsword.fbPrefs.getBoolPref(this.jsword.fbPrefs.DEBUG))
		return;
	this._logService.logStringMessage("FireBible: " + message);
};

FireBible.prototype.printStackTrace = function(e) {
	var traces = e.getStackTrace();
	for (var i = 0; i < traces.length; i++) {
		this.log(traces[i].toString());
	}
};

FireBible.prototype.openPrefs = function() {
	if (this.windowManager.getMostRecentWindow("firebible:preferences") == null) {
		window.openDialog('chrome://firebible/content/preferences.xul',
				'_blank');
	}
};

FireBible.prototype.loadErrorHTML = function(errorMsg) {
	this.jsword.error = errorMsg;
	// var url = "sword://status/";
	var url = "chrome://firebible/content/status.html";
	gBrowser.loadURI(url);
};

/**
 * This method is called from the toolbar when the user makes a valid selection.
 */
FireBible.prototype.loadHTML = function(bible, book, longName, chapter) {
	var url = "bible://" + bible + "/" + book;
	if (chapter != "ALL") {
		url += "." + chapter;
	}
	gBrowser.loadURI(url);
};

FireBible.prototype.changeIcon = function(browser, aTab, aURI) {
	// var browser = gBrowser.getBrowserForTab(aTab);

	if (aURI) {
		if (!(aURI instanceof Components.interfaces.nsIURI)) {
			var ios = Components.classes["@mozilla.org/network/io-service;1"]
					.getService(Components.interfaces.nsIIOService);
			aURI = ios.newURI(aURI, null, null);
		}
		alert('hello 3' + aURI + gBrowser.mFaviconService + ","
				+ browser.currentURI);
		// gBrowser.mFaviconService.setAndLoadFaviconForPage(browser.currentURI,
		// aURI, false);
		gBrowser.mFaviconService.setAndFetchFaviconForPage(browser.currentURI,
				aURI, false, gBrowser.mFaviconService.FAVICON_LOAD_NON_PRIVATE);
	}

	// gBrowser.updateIcon(aTab);
	gBrowser.updateIcon(gBrowser.tabContainer.selectedItem);

	for (var i = 0; i < gBrowser.mProgressListeners.length; i++) {
		var p = gBrowser.mProgressListeners[i];
		if ("onLinkIconAvailable" in p)
			try {
				p.onLinkIconAvailable(browser);
			} catch (e) {
				// don't inhibit other listeners
				Components.utils.reportError(e);
			}
	}
};

/**
 * This method is called from the protocol code (and ubiquity) to retrieve the
 * content. mode = 0 - default, for display in Firefox browsing area mode = 1 -
 * ubiquity textarea or clipboard execution mode = 2 - ubiquity preview
 */
FireBible.prototype.load = function(uri, mode) {
	if (mode == null)
		mode = 0;
	var uriText = mode == 0 ? (uri.host + uri.path) : uri;
	uriText = decodeURI(uriText);
	var jSword = this.getJSword();
	var uriS = unescape(uriText);
	var bookKeyString = this.parseURL(uriS);
	return [ jSword.getHTML(bookKeyString[0], bookKeyString[1], mode),
			bookKeyString ];
};

FireBible.prototype.getSelectedBible = function() { //FIXME not used anymore
	var bible = this.bibleControl.bible;
	return bible;
};

FireBible.prototype.isIndexed = function(bible) {
	return this.jsword.utilKlass.isIndexed(bible);
};

FireBible.prototype.search = function(bible, search) {
	this.log("searching for " + search);
	try {
		this.jsword.pageTitle = this._("searchResults.title", [ search ]);
		var key = this.jsword.utilKlass.find(bible, search, this.jsword.cl);
		if (!key || key == "") {
			this.loadErrorHTML(this._("noResults.text"));
			return;
		}
		var uri = "bible://" + key;
		gBrowser.loadURI(uri);
	} catch (e) {
		e = this.jsword.getRootException(e);
		this.promptService.alert(window, this._("error.title"), this._(
				"searching.error", [ e ]));
		this.log(e);
		this.printStackTrace(e);
	}
};

FireBible.prototype.index = function(bible, stack, bibleControl) {
	this.bibleControl = bibleControl;
	this.progressMeter = stack.firstChild;
	this.indexLabel = stack.lastChild.firstChild;
	this.progressMeter.value = 0;
	this.indexLabel.value = "";
	this.iManager = this.jsword.utilKlass.index(bible);
	this.indexInterval = setInterval("fireBible.checkIndexStatus();", 250);
};

FireBible.prototype.notify = function(message) {
	this.observerService.notifyObservers(this, this.name, message);
};

FireBible.prototype.observe = function(subject, topic, data) {
	if (data == this.getJSword().fbPrefs.MODULE_DIRS) {
		this.log("module directory list modified");
		// not reacting to observer for this case because the user could
		// manually request a reload as well (perhaps by adding more to an *existing* directory) 
		// and we would like to handle both cases with the same flow
//		this.initBibleList(true, false);
//		this.reloadBooksIfRequired(true);
	} else if (data == this.getJSword().fbPrefs.INCLUDE_COMM) {
		this.initBibleList(false, true);
	} else if (data == FireBible.prototype.BOOKS_RELOADED) {
		this.initBibleList(true, false);		
	}
};

FireBible.prototype.checkIndexStatus = function() {
	var details = this.iManager.getIndexStatus();
	this.progressMeter.value = details[1]; // work completed
	this.indexLabel.value = details[2]; // current work section
	var fin;
	if (details[0].getClass) {
		fin = details[0].booleanValue();
	} else {
		fin = details[0];
	}
	if (fin) {
		clearInterval(this.indexInterval);
		this.bibleControl.resetSearch();
		if (details[3] == null && this.isIndexed(this.iManager.getBook())) {
			this.bibleControl.doSearch(null, true);
		} else {
			this.promptService.alert(window, this._("error.title"), this._(
					"indexing.error", [ this.jsword
							.getRootException(details[3]) ]));
		}
	}
};

FireBible.prototype.manageBooks = function(win) {
	if (this.initUtils()) {
		xulSword.addNewModule(win);
		//this.booksNeedReloading = true;
	}
};

FireBible.prototype.reloadBooksIfRequired = function(force) {
	if (!this.booksNeedReloading && !force) return;
	LibSword.quitLibsword();
	this.jsword.initLibsword();
//	this.initBibleList(true, false);
	this.notify(FireBible.prototype.BOOKS_RELOADED);
	this.booksNeedReloading = false;
}

FireBible.prototype.suggestBooks = function(text) {
	var books = new Array();
	// get the book name from the text
	var temp = this.verseEx.exec(text);
	if (temp == null || temp.length < 2)
		return books;
	text = this.trim(temp[1]);
	var jsword = this.getJSword();
	if (!jsword || !jsword.utilsInitialized) {
		return books;
	}
	var booklist = this.getJSword().getBookList();
	if (booklist == null) {
		return books;
	}
	for (var i = 0; i < booklist.length; i++) {
		var name = booklist[i].name;
		if (name.match(text, "i")) {
			var mStart = (name.match("^" + text, "i") != null);
			var score = text.length / name.length;
			books.push([ name, booklist[i].sName, mStart, score ]);
		}
	}

	if (books.length == 0) {
		return books;
	}

	if (temp.length > 2) {
		var verses = this.trim(temp[2]);
		if (verses.length > 0) {
			var finalBooks = new Array();
			var bible = this.jsword.getDefaultBible();
			for (var i = 0; i < books.length; i++) {
				var key = books[i][1] + "." + verses;
				var bc = LibSword.parseVerseKey(bible, key).split(",");
				var valid = ((parseInt(bc[4]) != -1) //chapter is valid 
						&& (parseInt(bc[3]) <= parseInt(bc[4]))); //last verse is <= max verses
				this.log("bc is " + bc + " valid is " + valid + "," + bc[4] + "," + parseInt(bc[4]));				
				//var valid = this.jsword.utilKlass.getReference(key);
				if (valid) {
					finalBooks.push([ books[i][0] + " " + verses, books[i][1],
							books[i][2], books[i][3] ]);
				}
			}
			if (finalBooks.length == 0) {
				return null;
			} else {
				return finalBooks;
			}
		}
	}
	return books;
};

FireBible.prototype.suggestBibles = function(text) {
	var bibles = new Array();
	var jsword = this.getJSword();
	if (!jsword || !jsword.utilsInitialized) {
		return bibles;
	}
	for (var i = 0; i < jsword.bibleList.length; i++) {
		var bible = jsword.bibleList[i];
		if (bible.config.match(text, "i") || bible.description.match(text, "i")) {
			var mStart = (bible.config.match("^" + text, "i") || bible.description.match("^"
					+ text, "i"));
			bibles.push([ bible.config, null, mStart ]);
		}
	}
	return bibles;
}

FireBible.prototype.trim = function(text) {
	return text.replace(/^\s+|\s+$/g, '');
};

FireBible.prototype._ = function(key, args) {
	if (!args) {
		return this.strings.getString(key);
	} else {
		return this.strings.getFormattedString(key, args);
	}
};

var fireBible = new FireBible();
fireBible.init();

window.addEventListener("load", function() {
	fireBible.onLoad();
}, false);
window.addEventListener("unload", function() {
	fireBible.onUnload();
}, false);
