<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="1.0">

	<!-- Version 3.0 is necessary to get br to work correctly. -->
	<xsl:output method="html" version="3.0"
		omit-xml-declaration="yes" indent="no" />
		
	<xsl:param name="versemode"/>
	<xsl:param name="verseline"/>
	<xsl:param name="versess"/>

	<xsl:template match="@*|node()">
		<xsl:copy>
			<xsl:apply-templates select="@*|node()" />
		</xsl:copy>
	</xsl:template>
	
	<!--  mods to show book, chapter and verse numbers -->
	<xsl:template match="span[starts-with(@class, 'vs')]">
		<span>
		  <xsl:attribute name="class">
        <xsl:value-of select="@class"/>
      </xsl:attribute>
			<xsl:attribute name="title">
				<xsl:value-of select="@title"/>
			</xsl:attribute>
		</span>&#160;
		<xsl:if test="not($versess='true') and not($versemode=0)">(</xsl:if>
		<xsl:if test="$versemode=3"> <!--  book chapter:verse -->
			<sup class = "bookname"><xsl:value-of select="substring-before(@title,'.')"/>&#160;</sup>
		</xsl:if>
		<xsl:if test="$versemode=2 or $versemode=3"> <!--  chapter:verse -->
			<sup class = "chapternum"><xsl:value-of select="substring-before(substring-after(@title,'.' ),'.')"/>:</sup>
		</xsl:if>
		<xsl:apply-templates select="@*|node()" />
		<xsl:if test="$verseline='true'">
			<br/>
		</xsl:if>
	</xsl:template>
	<xsl:template match="sup[@class='versenum']">
		<!--  verse number is generated by xulsword by default, prevent display if mode is 0 -->
		<xsl:if test="not($versemode=0)">
			<sup class="versenum"><xsl:value-of select="."/></sup>
		</xsl:if>
		<xsl:if test="not($versess='true') and not($versemode=0)">)</xsl:if>		
	</xsl:template>
	
	<xsl:template match="div[@class='nlist']">
		<!-- create anchor -->
		<p><a>
			<xsl:attribute name='id'>
				<xsl:value-of select="@title"/>
			</xsl:attribute>
		</a>
		<xsl:variable name = "noBook">
			<xsl:call-template name="substring-before-last">
				<xsl:with-param name="input" select="substring-after(substring-after(@title,'.'), '.')"/>
				<xsl:with-param name="substr" select="'.'"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:call-template name="convert-reference">
			<xsl:with-param name="reference" select="$noBook"/>
		</xsl:call-template>
		<br/>
		
		<!--  footnote or crossreference? -->		
		<xsl:variable name="listType" select="substring-before(@title,'.')"/>
		<xsl:if test="$listType='cr'">
			<xsl:call-template name="splitRefs">			
				<xsl:with-param name="references" select="."/>				
			</xsl:call-template>
		</xsl:if>
		<xsl:if test="$listType='fn'">
			<xsl:value-of select="."/>
		</xsl:if>
		</p>
	</xsl:template>
	
	<!-- Strong's numbers -->
	<xsl:template match="span[starts-with(@class, 'sn')]">
		<div style="display: inline;">
			<span style="display: table-row">			
				<xsl:value-of select="."/>
			</span>
			<span class = "sn">
				<xsl:call-template name="splitStrongs">
					<xsl:with-param name="slist" select="substring-after(@class, ' ')"/>
				</xsl:call-template>		
			</span>			
		</div>
	</xsl:template>
	
	<xsl:template name="splitStrongs">
		<xsl:param name="slist"/>
		<xsl:choose>		
			<xsl:when test="not(contains($slist,' '))">
				<xsl:choose>
					<xsl:when test="starts-with($slist, 'S')">
						<a href="{concat(concat('javascript:loadDict(&quot;', substring($slist, 3)), '&quot;)')}">
							<xsl:value-of select="substring($slist, 4)"/>
						</a>
					</xsl:when>
					<xsl:otherwise>
						<a href="{concat(concat('javascript:loadDict(&quot;', substring($slist, 2)), '&quot;)')}">
							<xsl:value-of select="substring($slist, 4)"/>
						</a>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="splitStrongs">
					<xsl:with-param name="slist" select="substring-before($slist, ' ')"/>
				</xsl:call-template>
				&#160;
				<xsl:call-template name="splitStrongs">
					<xsl:with-param name="slist" select="substring-after($slist, ' ')"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!--  handle links in DBD -->
	<xsl:template match="span[@class='sr']">
		<a href="{concat('bible://', @title)}">
			<xsl:value-of select="node()"/>
		</a>	
	</xsl:template>
	
	<!--  handle cross references  -->
	<xsl:template match="span[@class='cr']|span[@class='fn']">
		<a href="{concat('#', concat(@class, concat('.', @title)))}">
			<xsl:copy-of select="."/>
		</a>	
	</xsl:template>
	
	<xsl:template name = "splitRefs">
		<xsl:param name="references"/>
		<xsl:choose>		
			<xsl:when test="not(contains($references,';'))">
				<xsl:call-template name="createLink">
					<xsl:with-param name="reference" select="$references"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="createLink">
					<xsl:with-param name="reference" select="substring-before($references, ';')"/>
				</xsl:call-template>				
				&#160;
				<xsl:call-template name="splitRefs">
					<xsl:with-param name="references" select="substring-after($references, ';')"/>					
				</xsl:call-template>				
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="createLink">
		<xsl:param name="reference"/>
		<a>
			<xsl:attribute name='href'>
				 <xsl:value-of select="normalize-space($reference)"/>
			</xsl:attribute>
			<xsl:call-template name="convert-reference">
				<xsl:with-param name="reference" select="normalize-space($reference)"/>
			</xsl:call-template>
		</a>
	</xsl:template>
	
	<xsl:template name="substring-before-last">
		<xsl:param name="input" />
		<xsl:param name="substr" />
		<xsl:if test="$substr and contains($input, $substr)">
			<xsl:variable name="temp" select="substring-after($input, $substr)" />
			<xsl:value-of select="substring-before($input, $substr)" />
			<xsl:if test="contains($temp, $substr)">
				<xsl:value-of select="$substr" />
				<xsl:call-template name="substring-before-last">
					<xsl:with-param name="input" select="$temp" />
					<xsl:with-param name="substr" select="$substr" />
				</xsl:call-template>
			</xsl:if>
		</xsl:if>		
	</xsl:template>
	
	<xsl:template name="convert-reference">
		<xsl:param name="reference"/>
		<xsl:choose>
			<xsl:when test="not(contains($reference,'-'))">
				<xsl:variable name="chapVerse" select="substring-after($reference, '.')"/>
				<xsl:value-of select="substring-before($reference, '.')"/>&#160;<xsl:value-of select="substring-before($chapVerse, '.')"/>:<xsl:value-of select="substring-after($chapVerse, '.')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="convert-reference">
					<xsl:with-param name="reference" select="substring-before($reference, '-')"/>
				</xsl:call-template>-<xsl:value-of select="substring-after(substring-after(substring-after($reference, '-'), '.'), '.')"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	
</xsl:stylesheet>
