/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008-2015 Brian Fernandes <brian@thegoan.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
JSword.prototype.setComponentLocation = function(cLocation) {
//	this.cLocation = this.getFileURI(clocation);
	this.cLocation = cLocation.parent.parent.path;
};

JSword.prototype.initUtils = function(silent) {
	if (this.utilsInitialized) return true;
	this.log("Initializing JSword Utils");
	xulSword = new XULSword(this.cLocation);
	if (!xulSword.startxulsword(xulSword)) {
		fireBible.promptService.alert(window, "Error Downloading LibSword", "FireBible cannot be used without completing the LibSword library download.");
		return false;
	} else {
		this.initLibsword();
		this.getBibleList(false, false);
		this.pageTitle = null;	
		this.utilsInitialized = true;
		return this.utilsInitialized;
	}
};

JSword.prototype.initLibsword = function() {
	LibSword.initLibsword(this.cLocation, this.fbPrefs.getModuleDirs());
	xulSword.xulswordInit();
	this.libSword = LibSword;
	this.fbPrefs.initSwordPrefs();
	this.bibleList = null;
};

//copied from dictTexts.js#read
JSword.prototype.getDictionaryKeys = function(module) {
	// the key list is cached because it can take several seconds to
	// process large dictionaries!
	if (!this.keyList[module]) {
		this.keyList[module] = LibSword.getAllDictionaryKeys(module).split("<nx>");
		this.keyList[module].pop();
		this.sortOrder = LibSword.getModuleInformation(module, "LangSortOrder");
		if (this.sortOrder != XULSword.prototype.NOTFOUND) {
			this.sortOrder += "0123456789";
			this.langSortSkipChars = LibSword.getModuleInformation(module, "LangSortSkipChars");
			if (this.langSortSkipChars == XULSword.prototype.NOTFOUND) this.langSortSkipChars = "";
			this.keyList[module].sort(this.dictSort);
		}
	}
	return this.keyList[module];
};

JSword.prototype.getDictionaryValue = function(module, key) {
//    var ret = { htmlList:"", htmlHead:Texts.getPageLinks(), htmlEntry:"", footnotes:null };
    
	var keyList = this.getDictionaryKeys(module);    
    
    // get actual key
    if (!key) key = keyList[0];    
    if (key == "DailyDevotionToday") {
      var today = new Date();
      key = (today.getMonth()<9 ? "0":"") + String(today.getMonth()+1) + "." + (today.getDate()<10 ? "0":"") + today.getDate();
    }
    
    // get htmlEntry
    var de = LibSword.getDictionaryEntry(module, key);
    
    return "<div class=\"dictentry\">" + de + "</div>";
};

JSword.prototype.isInitialized = function() {
	return this.utilsInitialized;
};

JSword.prototype.getModules = function(rType, rFeature, matchClass) {
	var config;
	if (this.moduleList == null) {
		this.moduleList = [];
		var config;
		for (config in XULSword.prototype.ModuleConfigs) {
			var description = LibSword.getModuleInformation(config, "Description");
			var feature = LibSword.getModuleInformation(config, "Feature") + " / " + LibSword.getModuleInformation(config, "Category");
			var fbType = this.getFBClassCategory(xulSword.getModuleLongType(config), feature);
			var bArray = {};
			bArray.config = config;
			bArray.description = description;
			bArray.bClass = fbType[0];
			bArray.bCat = fbType[1];
			bArray.feature = feature;
			bArray.direction = XULSword.prototype.ModuleConfigs[config].direction;
			bArray.version = LibSword.getModuleInformation(config, "Version"); 
			this.moduleList.push(bArray);
		}
	}
	
	if  (rType != null) {
		var fModuleList = [];		
		for (var i = 0; i < this.moduleList.length; i++) {
			var mod = this.moduleList[i];
			var matchWith = matchClass ? mod.bClass : mod.bCat;
			if (matchWith == rType) {
				if (rFeature != null && mod.feature.search(rFeature) == -1) {
					continue;
				} 
				fModuleList.push(mod);
			}
		}
		return fModuleList;
	}
	
	return this.moduleList;
};

JSword.prototype.getFBClassCategory = function(newType, feature) {
	if (feature == null) feature = "";
	var bClass;
	switch (newType) {
		case xulSword.BIBLE:			
		case "bible":
			if (feature.search("Cults / Unorthodox / Questionable Material") != -1) {
				bClass = "questionable";
			} else {
				bClass = "bible";
			}
			return [bClass, "bible"];		
		case xulSword.GENBOOK:
		case "genbook":
			if (feature.search("Essays") != -1) {
				bClass = "essays";
			} else if (feature.search("Maps") != -1) {
				bClass = "maps";
			} else {
				bClass = "genbook";
			}
			return [bClass, "genbook"];
		case xulSword.DICTIONARY:
		case "dictionary":
			if (feature.search("Glossary") != -1 || 
						feature.search("Glossaries") != -1) {
				bClass = "glossary";
			} else if (feature.search("Daily Devotional") != -1 || 
						feature.search("DailyDevotion") != -1) {
				bClass = "dailydevotional";
			} else {
				bClass = "dictionary";
			}
			return [bClass, "dictionary"];
		case xulSword.COMMENTARY:
		case "commentary":
			return ["commentary", "bible"];
		default:
			return ["other", "bible"];
	}
};

JSword.prototype.getDictionaryModules = function(type) {
	var def = (type == 0) ? "GreekDef" : ((type == 1) ? "HebrewDef" : "GreekParse");
	return this.getModules("dictionary", def);
};

JSword.prototype.isDictionaryModule = function(module, type) {
	var moduleArray = this.getModuleArray(module);
	if (moduleArray != null) {
		var def = (type == 0) ? "GreekDef" : ((type == 1) ? "HebrewDef" : "GreekParse");
		return (moduleArray.bCat == "dictionary" && moduleArray.feature.search(def) != -1);
	}
	return false;
};

JSword.prototype.isCommentaryModule = function(module) {
	var moduleArray = this.getModuleArray(module);
	if (moduleArray != null) {
		return (moduleArray.bClass == "commentary");
	}
	return false;
};

JSword.prototype.getBibleList = function(forceReset, doNotWarn) {
	if (this.bibleList == null || forceReset) {
		this.moduleList = null;
		this.bibleList = this.getModules("bible", null);
	}
	if (this.bibleList.length == 0 && !doNotWarn) {
		/*var listener = {
				observe: function(subject, topic, data) {
					if (topic == "alertclickcallback") {
						gBrowser.loadURI("http://thegoan.com/firebible#use");
					}
				}
		}
		Components.classes['@mozilla.org/alerts-service;1']
		.getService(Components.interfaces.nsIAlertsService)
		.showAlertNotification("chrome://firebible/skin/bible48.png", "No Modules Found", 
				"FireBible has been successfully installed but you do not have any modules installed. Click here for details.", 
				false, null, null);*/
		
		var box = gBrowser.getNotificationBox();
		var notification = box.getNotificationWithValue('firebible-notification');
		if (notification == null) {
			var button = {
				label: 'Help',
				accessKey: 'H',
				popup: null,
				callback: function() { gBrowser.loadURI("http://thegoan.com/firebible#modules"); }
			};
			box.appendNotification("FireBible has been successfully installed but you do not have any modules installed. Click Help for assitance.", 
					"firebible-notification",
					"chrome://firebible/skin/bible16.png",
					box.PRIORITY_WARNING_MEDIUM, [button]);
		}
	}
	if (this.bibleList.length > 0) {
		var box = gBrowser.getNotificationBox();
		var notification = box.getNotificationWithValue('firebible-notification');
		if (notification != null) {
			notification.close();
		}
	}
	return this.bibleList;
};

JSword.prototype.getBookList = function(bible) {
	if (bible == null) {
		if (this.bibleList.length > 0) {
			bible = this.bibleList[0].config;
		} else {
			return null;
		}
	}
	this.bookList = new Array();
	var bookList = LibSword.getModuleBooks(bible);
	var books = bookList.split("<nx>");
	for (var i=0; i < books.length; i++) {
		var book = books[i];
		var parts = book.split("<bk>");
		this.bookList[i] = new Object();
		this.bookList[i].name = parts[0];
		this.bookList[i].sName = parts[1];
		this.bookList[i].chapters = parseInt(parts[2]);
	}	
	return this.bookList;
};

JSword.prototype.getTitle = function(module, key) {
	/*var pageTitle = "";	
	var module;
	if (Array.isArray(modules)) {
		for (var i=0; i < modules.length; i++) {
			pageTitle += "," + modules[i];
		}
		pageTitle = pageTitle.substring(1) + ": ";
		module = modules[0];
	} else {
		module = modules;
		pageTitle = modules + ": ";
	}*/	
	var bc = LibSword.parseVerseKey(module, key).split(",");
	var pageTitle = "";
	pageTitle += bc[0] + " " + bc[1];
	if (!(bc[2] == "1" && bc[3] == bc[4])) {
		if (bc[2] == bc[3]) {
			pageTitle += ":" + bc[2];
		} else {
			pageTitle += ":" + bc[2] + "-" + bc[3];
		}
	}
	return pageTitle;
};

/*
 * This method should only be called by FireBible#load
 *
 * mode = 0 - default, for display in Firefox browsing area
 * mode = 1 - ubiquity textarea or clipboard execution
 * mode = 2 - ubiquity preview
 */
JSword.prototype.getHTML = function(bibles, key, mode) {
	var ubiquity = mode >= 1;
	try {
		/*if (key == "status/") {
			return this.utilKlass.getErrorHTML(this.error);
		}
		if (bibles == null) {
			bibles = this.getDefaultBible();
		}
		var bibleA = bibles.split(".");
		this.log("Bible is " + bibles);*/		
		key = key.split("#")[0];
		this.log("Key is " + key);
		var maxVerses = (mode != 2) ? -1
									: this.fbPrefs.getIntPref(this.fbPrefs.MAX_VERSES);
		
		var bibleA = bibles;
		
		var chapterText;
		var titleString = "";
		var headingString = "";
		var notes = null;
		var prefs = this.fbPrefs.getDisplayPrefs(mode, (bibleA.length > 1));		
		var css = this.fbPrefs.getDisplayCSS(ubiquity);
		var pageTitle = null;
		var chapterTitle = null;
		var contentClass = "content";
		if (bibleA.length == 1)  {
			var moduleArray = this.getModuleArray(bibleA[0]);
			var module = moduleArray.config;
			var type = moduleArray.bCat;
			if (type == "bible") {
				chapterText = LibSword.getChapterText(module, key);
				chapterTitle = this.getTitle(module, key);
				pageTitle = module + ": " + chapterTitle;
				if (prefs[1]) {
					notes = LibSword.getNotes();
				}
			} else if (type == "genbook") {
				chapterText = LibSword.getGenBookChapterText(module, "/" + key);
			} else if (type == "dictionary") {
				chapterText = this.getDictionaryValue(module, key);
			}
			
			if (moduleArray.direction == "rtl") {
				contentClass += " RTL";
			}
		} else {
			var moduleList = "";
			for (var i = 0; i < bibleA.length; i++) {
				var module = this.getModule(bibleA[i]);
				moduleList += "," + module;
			}
			moduleList = moduleList.substring(1);
			chapterText = LibSword.getChapterTextMulti(moduleList, key, false);
			chapterTitle = this.getTitle(this.getModule(bibleA[0]), key);
			pageTitle = moduleList + ": " + chapterTitle;
		}
		
		
		if (notes) {
			notes = this.applyXSLT(bibleA[0], "<div>" + notes + "</div>", ubiquity);
		}

		chapterText = this.applyXSLT(bibleA[0], "<div>" + chapterText + "</div>", ubiquity);
				
		if (pageTitle != null) {
			titleString = "\n\t\t<title>" + pageTitle + "</title>";			
			if (this.fbPrefs.getBoolPref(this.fbPrefs.SHOW_HEADINGS, ubiquity)) {
				headingString = "<h2 class = 'head1'>" + chapterTitle + "</h2>";
			}
		}
		
		var html = "\<!DOCTYPE html>\n\
<html>\
\n\t<head>" + titleString + "\
\n\t\t<link rel='stylesheet' href='resource://firebible/skin/viewport/texts/texts-htm.css' type='text/css'/>\
\n\t\t<link rel='stylesheet' href='resource://firebible/skin/common/global-htm.css' type='text/css'/>\
\n\t\t<link rel='stylesheet' href='resource://firebible/skin/dialogs/printPassage/printPassage-htm.css' type='text/css'/>\
\n\t\t<link rel='stylesheet' href='resource://firebible/skin/viewport/layout.css' type='text/css'/>\
\n\t\t<script src='resource://firebible/content/firebible-utils.js'/></script>\
\n\t<style type='text/css'>" + css + "</style>" +
"\n\t</head>\
\n\t<body>";
if (notes) {
	html += "\n\t\t<nav class='notes'>" + notes + "\n\t\t</nav>"; 
} 
html += "\
\n\t\t<section class='" + contentClass + "'>" + headingString + chapterText + "\n\t\t</section>" +
//"<script>$('.content').flowtype({minimum: 500, maximum: 1200, maxFont: 40, minFont: 12});</script>" +
"\n\t</body>\n</html>";		
		return html;
	} catch (e) {
		this.log(this.getRootException(e));
	}
};


JSword.prototype.applyXSLT = function(module, text, ubiquity) {
	if (text.indexOf("<br>") != -1) { //basic failsafe, document will not be well formed in this case
		text = text.replace(/<br>/g,"<br/>");
	}
	if (text.indexOf("<img") != -1) {
		this.fbPrefs.setCharPref("capability.policy.swordlinks.checkloaduri.enabled", "allAccess");
		var policyNames = this.fbPrefs.getCharPref("capability.policy.policynames");
		if (!policyNames || policyNames.indexOf("swordlinks") == -1) {
			//needs addition
			if (!policyNames) {
				policyNames = "swordlinks";
			} else {
				policyNames = policyNames + ",swordlinks"; 
			}
			this.fbPrefs.setCharPref("capability.policy.policynames", policyNames);
		}
		var domains = "sword://" + module + " bible://" + module + " sword://" + (new Date()).getTime(); //this ensures that this property is *changed* at least once per session - if not, it doesn't take effect
		this.fbPrefs.setCharPref("capability.policy.swordlinks.sites", domains);
		
		
		text = text.replace(/<img(.*?)src\s*=\s*['"](.*?)['"](.*?)>/g, function(match, p1, p2, p3) {
			var endString;
			if (p3 && p3.match(/\/\s*$/)) {//ends with a />
				endString = ">";
			} else {
				endString = "/>";
			}			
			return "<img" + p1 + "src=\"" + unescape(p2) + "\"" + (p3 ? p3 : "") + endString;
		});		
		text = text.replace(/<\/img>/g, ""); // remove closing </img> in case it was well formed already
	}
	
	if (!this.xsltProcessor) {
		this.xsltProcessor = Components.classes["@mozilla.org/document-transformer;1?type=xslt"].createInstance(Components.interfaces.nsIXSLTProcessor);		
		var xmlDoc = document.implementation.createDocument("", "test", null);
		xmlDoc.async = false;
		xmlDoc.load('chrome://firebible/content/firebible.xsl');		
		this.xsltProcessor.importStylesheet(xmlDoc);
		this.domParser = Components.classes["@mozilla.org/xmlextras/domparser;1"].createInstance(Components.interfaces.nsIDOMParser);
		this.domSerializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"].createInstance(Components.interfaces.nsIDOMSerializer);
	}
		
	var doc = this.domParser.parseFromString(text, "application/xml");
	this.xsltProcessor.setParameter(null, "versemode", this.fbPrefs.getIntPref(this.fbPrefs.VERSE_FORMAT, ubiquity));
	this.xsltProcessor.setParameter(null, "verseline", this.fbPrefs.getBoolPref(this.fbPrefs.VERSE_SEPARATE_LINE, ubiquity));
	this.xsltProcessor.setParameter(null, "versess", this.fbPrefs.getBoolPref(this.fbPrefs.VERSE_SUPERSCRIPT, ubiquity));
	var doc2 = this.xsltProcessor.transformToDocument(doc);	
	return this.domSerializer.serializeToString(doc2);	
};

JSword.prototype.getModuleArray = function(moduleName) {
	this.getModules();
	//this.log("caller is " + arguments.callee.caller.toString());
	moduleName = moduleName.toUpperCase();
	var element = "";
	for (var i = 0; i < this.moduleList.length; i++) {
		var modArray = this.moduleList[i];
		var config = modArray.config.toUpperCase();
		element += "," + config;
		if (config == moduleName) {			
			return modArray;
		}
	}
	return null;
};

JSword.prototype.getModule = function(moduleName) {
	var modArray = this.getModuleArray(moduleName);
	if (modArray != null) {
		return modArray.config;
	}	
	return null;
};



JSword.prototype.getDefaultBible = function() {
	var bible;
	//get the default bible
	var defaultBible = this.fbPrefs.getCharPref(this.fbPrefs.DEFAULT_BIBLE);
	if (this.bibleList && defaultBible && defaultBible.length > 0) {
		for (var i = 0; i < this.bibleList.length; i++) {
			var bibleA = this.bibleList[i];
			if (bibleA.config == defaultBible) {
				bible = bibleA.config;
				break;
			}
		}
	}
	if (bible == null && this.bibleList && this.bibleList.length > 0) {
		for (var i = 0; i < this.bibleList.length; i++) {
			var temp = this.bibleList[i];
			if (temp.bClass != "commentary") {
				bible = temp.config;
				break;
			}
		}
	}
	return bible;
};

JSword.prototype.getRootException = function(e) {
	while(e != null) {
		if (!e.getClass) {	//with the new Java plugin, e can be a JS string :|
			return e;
		}
		var eClass = e.getClass().getName();
		if (eClass == "java.security.PrivilegedActionException") {
			e = e.getCause();
		} else if (eClass == "java.lang.reflect.InvocationTargetException") {
			e = e.getTargetException();
		} else {
			return e;
		}
	}
	return null;
};

JSword.prototype.getFileURI = function(path) {
	path = escape(path.replace(/\\/g,'/'));
	return "file://" + ((path.charAt(0) == '/') ? path : ("/" + path));
};

JSword.prototype.log = function(message) {
	if (!this.fbPrefs.getBoolPref(this.fbPrefs.DEBUG)) return;
	this._logService.logStringMessage("FireBible: " + message);
};

JSword.prototype.showLog = function(silent, message, isWarning) {
	if (!silent) {
		this.promptService.alert(window, this._(!isWarning ? "error.title"
														   : "warning.title"),
														   message);
	}
	this.log(message);
};


JSword.prototype.getFireBible = function() {
	var win = this.windowManager.getMostRecentWindow("navigator:browser");
	return (win != null) ? win.fireBible : null;
};

JSword.prototype.notifyFireBible = function(data) {
	var fb = this.getFireBible();
	if (fb != null) {
		fb.notify(data);
	}
};

JSword.prototype._ = function(key, args) {
	var fb = this.getFireBible();
	if (fb != null) {
		return fb._(key, args);
	}
	return "_string_not_found - " + key;
};

JSword.prototype.fixBooks = function() {
	if (this.promptService.confirm(window, this._("books.title"), this._("downloadBibles.message"))) {
		var fb = this.getFireBible();
		if (fb != null) {
			fb.manageBooks();
		}
	}
};

//copied from https://developer.mozilla.org/Talk:en/XPCOM_Interface_Reference/nsIZipReader
JSword.prototype.extractFiles = function(root, xpiFile) {

	var zipReader = Components.classes["@mozilla.org/libjar/zip-reader;1"]
			.createInstance(Components.interfaces.nsIZipReader);
	zipReader.open(xpiFile);
	zipReader.test(null);
	// create directories first
	var entries = zipReader.findEntries("*/");
	while (entries.hasMore()) {
		var entryName = entries.getNext();
		var target = this.getItemFile(root, entryName);
		if (this.firstFolder == null) {
			this.firstFolder = entryName;
		}
		if (!target.exists()) {
			try {
				target.create(Ci.nsILocalFile.DIRECTORY_TYPE, 0777);
			} catch (e) {
				this.log("extractExtensionsFiles: failed to create target directory for extraction "
						+ " file = "
						+ target.path
						+ ", exception = "
						+ e
						+ "\n");
			}
		}
	}

	entries = zipReader.findEntries(null);
	while (entries.hasMore()) {
		var entryName = entries.getNext();
		target = this.getItemFile(root, entryName);
		if (target.exists())
			continue;

		try {
			target.create(Ci.nsILocalFile.DIRECTORY_TYPE, 0777);
		} catch (e) {
			this.log("extractExtensionsFiles: failed to create target file for extraction "
					+ " file = " + target.path + ", exception = " + e + "\n");
		}
		zipReader.extract(entryName, target);
	}
	zipReader.close();
};

JSword.prototype.getItemFile = function(root, filePath) {
	var itemLocation = root.clone();
	var parts = filePath.split("/");
	for ( var i = 0; i < parts.length; ++i)
		itemLocation.append(parts[i]);
	return itemLocation;
};

JSword.prototype.QueryInterface = function(aIID) {
	if (!aIID.equals(nsIDownloader)) {
		throw Components.results.NS_ERROR_NO_INTERFACE;
	}
	return this;
};


function JSword() {
	const Ci = Components.interfaces;
	const Cc = Components.classes;
	this.promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].
                    		 getService(Ci.nsIPromptService);
	this.windowManager = Cc['@mozilla.org/appshell/window-mediator;1'].
												 getService(Ci.nsIWindowMediator);
	this._logService = Cc['@mozilla.org/consoleservice;1'].getService();
	this.fbPrefs = FireBible.prototype.firebible_prefs;
	this.fbPrefs.init(this);
	this.fbPrefs.initPrefs();
	FireBible.prototype.prefObserver.register(this.fbPrefs.getPrefBranch(), this);
	this.log("Loading JSword Utils");
	this.keyList = {};
};