/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008-2012 Brian Fernandes <brian@thegoan.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

function FireBible() {

}

FireBible.prototype.firebible_prefs = {};

FireBible.prototype.PREF_TRANS = {};

FireBible.prototype.firebible_prefs.init = function(jsword) {
	this.prefBranch = null;
	this.DEBUG = "extensions.firebible.debug.active";
	this.MODULE_DIRS = "extensions.firebible.module.directories";
	
	//default modules
	this.DEFAULT_BIBLE = "extensions.firebible.default.bible";
	this.DEFAULT_GREEK = "extensions.firebible.default.greek";
	this.DEFAULT_HEBREW = "extensions.firebible.default.hebrew";
	this.DEFAULT_GREEK_MORPH = "extensions.firebible.default.greek.morph";
	
	this.INCLUDE_COMM = "extensions.firebible.include.comm";
	//display preferences
	this.VERSE_SEPARATE_LINE = "extensions.firebible.verse.newline";
	this.VERSE_SUPERSCRIPT = "extensions.firebible.verse.superscript";
	this.VERSE_FORMAT = "extensions.firebible.verse.format";
	this.SHOW_HEADINGS = "extensions.firebible.show.headings";
	this.SHOW_NOTES = "extensions.firebible.show.notes";
	this.SHOW_STRONGS = "extensions.firebible.show.strongs";
	this.SHOW_MORPH = "extensions.firebible.show.morphology";
	this.UBIQUITY = "_ubiquity";
	
	//xulsword
	this.LIBSWORD_VERSION = "extensions.firebible.LibxulswordVersion";
	this.LIBSWORD_URL = "extensions.firebible.LibSwordURL";
	
	this.DEFAULT_ACTION = "extensions.firebible.default.action" + this.UBIQUITY;
	this.MAX_VERSES = "extensions.firebible.max.preview.verses" + this.UBIQUITY;
	this.ENABLE_UBIQUITY_INIT = "extensions.firebible.enable.init" + this.UBIQUITY;
	this.jsword = jsword;
	this.getPrefBranch();	

	FireBible.prototype.PREF_TRANS[this.SHOW_HEADINGS] = "Headings";
	FireBible.prototype.PREF_TRANS[this.SHOW_NOTES] = "Cross-references,Footnotes";
	FireBible.prototype.PREF_TRANS[this.SHOW_STRONGS] = "Strong's Numbers";
	FireBible.prototype.PREF_TRANS[this.SHOW_MORPH] = "Morphological Tags";	
};

FireBible.prototype.firebible_prefs.getPrefBranch = function() {
	if (!this.prefBranch) {
		this.prefBranch = Components.classes['@mozilla.org/preferences-service;1'].
		                    				  			getService().
		                    				  			QueryInterface(Components.interfaces.
		                    				  										 nsIPrefBranch);
	}
	return this.prefBranch;
};

FireBible.prototype.firebible_prefs.setCharPref = function(name, value) {
	this.getPrefBranch().setCharPref(name, value);
};

FireBible.prototype.firebible_prefs.getCharPref = function(name) {
	var value = null;
	try {
		value = this.getPrefBranch().getCharPref(name);
		if (value && value.length == 0) value = null;
	} catch (e) {
		this.jsword.log(e);
	}
	return value;
};

FireBible.prototype.firebible_prefs.setBoolPref = function(name, value) {
	this.getPrefBranch().setBoolPref(name, value);
};

FireBible.prototype.firebible_prefs.getBoolPref = function(name, ubiquity) {
	var suffix = ubiquity ? this.UBIQUITY : "";
	var value = null;
	try {
		value = this.getPrefBranch().getBoolPref(name + suffix);
	} catch (e) {
	}
	return value;
};

FireBible.prototype.firebible_prefs.setIntPref = function(name, value) {
	this.getPrefBranch().setIntPref(name, value);
};

FireBible.prototype.firebible_prefs.getIntPref = function(name, ubiquity) {
	var suffix = ubiquity ? this.UBIQUITY : "";
	var value = null;
	try {
		value = this.getPrefBranch().getIntPref(name + suffix);
	} catch (e) {
	}
	return value;
};

FireBible.prototype.firebible_prefs.getModuleDirs = function() {
	var dirs = this.getCharPref(this.MODULE_DIRS);
	if (!dirs || dirs.length == 0) return null;
	return dirs;
};

FireBible.prototype.firebible_prefs.getDisplayCSS = function(ubiquity) {
	var vCSS = null;
	var cCSS = null;
	var bCSS = null;
	var verseFormat = this.getIntPref(this.VERSE_FORMAT, ubiquity);
	switch (verseFormat) {
		case 0: //hide everything
			vCSS = "display: none;";
			//fall through
		case 1: //show only verse numbers
			cCSS = "display: none;";
			bCSS = "display: none;";
			break;			
		case 2: //show verse and chapter
			bCSS = "display: none;";
			break;
		case 3: //show everything
			break;
	}
	
	var css = "";
	if (this.getBoolPref(this.VERSE_SEPARATE_LINE, ubiquity)) {
		css += ".vs {display: block;}";
	}
	if (!vCSS &&  //verse is being displayed
			!this.getBoolPref(this.VERSE_SUPERSCRIPT, ubiquity)) {
		vCSS = "font-size: 1em; vertical-align: inherit;";
		if (!bCSS) bCSS = "font-size: 1em; vertical-align: inherit;";
		if (!cCSS) cCSS = "font-size: 1em; vertical-align: inherit;";
		
		css += ".spacer:after { content: ' ('; } .spacer {font-size: 1em; vertical-align: inherit;}";
		css += ".versenum:after { content: ') '; }";
	}

	if (vCSS) css += ".versenum { " + vCSS + " }\n";
	if (cCSS) css += ".chapternum {" + cCSS + " }\n";
	if (bCSS) css += ".bookname {" + bCSS + " }\n";
	
	if (!this.getBoolPref(this.VERSE_SUPERSCRIPT, ubiquity)) {
		css = "sup {font-size: 1em; vertical-align: inherit;}";
	}
	return css;
};

FireBible.prototype.firebible_prefs.getDisplayPrefs = function(mode,  multiBible) {
	var ubiquity = mode >= 1;
	
	var showHeadings = this.getBoolPref(this.SHOW_HEADINGS, ubiquity);
	this.setSwordPref(this.SHOW_HEADINGS, showHeadings);
	
	var showNotes = (multiBible || mode == 2) ? false : this.getBoolPref(this.SHOW_NOTES, ubiquity);
	this.setSwordPref(this.SHOW_NOTES, showNotes);	
	
	var showStrongs = !ubiquity && this.getBoolPref(this.SHOW_STRONGS);
	this.setSwordPref(this.SHOW_STRONGS, showStrongs);
	
	var showMorph = !ubiquity && this.getBoolPref(this.SHOW_MORPH);
	this.setSwordPref(this.SHOW_MORPH, showMorph);	
	
	return [showHeadings, showNotes, showStrongs, showMorph];
};

FireBible.prototype.firebible_prefs.initPrefs = function() {
	this.initPref(this.VERSE_SUPERSCRIPT, "bool", true);
	this.initPref(this.VERSE_SEPARATE_LINE, "bool", false);
	this.initPref(this.SHOW_HEADINGS, "bool", true);
	this.initPref(this.SHOW_NOTES, "bool", true);
	this.initPref(this.VERSE_FORMAT, "int", 1);

	this.initPref(this.VERSE_SUPERSCRIPT + this.UBIQUITY, "bool", true);
	this.initPref(this.VERSE_SEPARATE_LINE + this.UBIQUITY, "bool", false);
	this.initPref(this.SHOW_HEADINGS + this.UBIQUITY, "bool", false);
	this.initPref(this.SHOW_NOTES + this.UBIQUITY, "bool", false);
	this.initPref(this.VERSE_FORMAT + this.UBIQUITY, "int", 1);
	this.initPref(this.DEFAULT_ACTION, "int", 0);
	this.initPref(this.MAX_VERSES, "int", 30);
	this.initPref(this.ENABLE_UBIQUITY_INIT, "bool", false);
	
	this.initPref(this.LIBSWORD_VERSION, "char", "1.2");
	this.initPref(this.LIBSWORD_URL, "char", "ftp://ibtrussia.org/pub/xulsword/lib/");
};

FireBible.prototype.firebible_prefs.setSwordPref = function(pref, value) {
	var swordPref = FireBible.prototype.PREF_TRANS[pref].split(",");
	for (var count in swordPref) {		
		LibSword.setGlobalOption(swordPref[count], value ? "On" : "Off");
	}
};

FireBible.prototype.firebible_prefs.initSwordPrefs = function() {
	for (var pref in FireBible.prototype.PREF_TRANS) {
		var ePref = this.getBoolPref(pref);
		this.setSwordPref(pref, ePref);
	}	
	//always show verse numbers
	LibSword.setGlobalOption("Verse Numbers", "On");
	LibSword.setGlobalOption("Morpheme Segmentation","On");
};

FireBible.prototype.firebible_prefs.initPref = function(name, type, dValue) {
	switch(type) {
		case "char":
			var ePref = this.getCharPref(name);
			if (!ePref || ePref.length == 0) {
				this.setCharPref(name, dValue);
			}
			break;
		case "bool":
  		var ePref = this.getBoolPref(name);
  		if (ePref == null) {
  			ePref = dValue;
    		this.setBoolPref(name, ePref);
    	}
  		break;
  	case "int":
  		var ePref = this.getIntPref(name);
  		if (ePref == null) {
    		this.setIntPref(name, dValue);
    	}
  		break;
	}
};



FireBible.prototype.prefObserver = {
  register: function(prefBranch, jsword) {
    this._branch = prefBranch;
    this.jsword = jsword;
    this._branch.addObserver("", this, false);
  },

  unregister: function() {
    if(!this._branch) return;
    this._branch.removeObserver("", this);
  },

  observe: function(aSubject, aTopic, aData) {
    if(aTopic != "nsPref:changed") return;
    // aSubject is the nsIPrefBranch we're observing (after appropriate QI)
    // aData is the name of the pref that's been changed (relative to aSubject)
    switch (aData) {
      case this.jsword.fbPrefs.MODULE_DIRS:
      case this.jsword.fbPrefs.INCLUDE_COMM:
      	this.jsword.notifyFireBible(aData);
    	break;
    }
  }
};
