/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008-2011 Brian Fernandes <brian@thegoan.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
var mainWindow;
var fb;

function init_sidebar() {
	mainWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                   .getInterface(Components.interfaces.nsIWebNavigation)
                   .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                   .rootTreeItem
                   .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                   .getInterface(Components.interfaces.nsIDOMWindow);
	fb = mainWindow.fireBible;
	fbItemList.init();
	fbTreeView.init();
	gNavigatorBundle = document.getElementById('bundle_browser');
	fb.loadDictItem();
}

function bookmarkThisPage(browser) {
	//sets the "load in sidebar" flag
	var uri = browser.currentURI;
	var itemId = PlacesUtils.getMostRecentBookmarkForURI(uri);
	var existing = (itemId != -1);
	window.top.PlacesCommandHook.bookmarkPage(browser, PlacesUtils.bookmarksMenuFolderId, true);
	if (!existing) { //only if bookmark did not exist at the start
		var newItemId = PlacesUtils.getMostRecentBookmarkForURI(uri);
		if (newItemId != -1) {
			//bookmark was created (user could have clicked cancel)
			var txn = PlacesUIUtils.ptm.setLoadInSidebar(newItemId, true);
			PlacesUIUtils.ptm.doTransaction(txn);
		}
	}
}

var fbItemList = {

	init : function() {
		this.listbox = document.getElementById("firebible-itemlist");
		if (fb.initUtils(true)) {
			this.jSword = fb.getJSword();
			this.fillItemList();
		} else {
			mainWindow.toggleSidebar('viewFireBibleSidebar');
		}
	},

	fillItemList : function() {
		this.modules = new Array();
		this.bookIndexMap = new Object();
		var bibleList = this.jSword.getModules();
		var mCount = 0;
		for (var i = 0; i < bibleList.length; i++) {
			var bible = bibleList[i];
			if (bible.bClass == "bible" || bible.bClass == "questionable") continue;
			this.modules[mCount] = bible.config;
			this.bookIndexMap[bible.config] =  mCount++;//listItem;
			var listItem = this.listbox.appendItem(bible.description);
			listItem.setAttribute("class","listitem-iconic " + bible.bClass);
//			fb.log(bible.description + " Class: " + bible.bClass + ", Category: " + bible.bCat);
			var type = (bible.bClass == "commentary" || bible.bClass == "dailydevotional") ? bible.bClass : bible.bCat;
			listItem.setAttribute("type", type);			
		}
	},

	selectionChanged : function() {
		var module = this.modules[this.listbox.currentIndex];
		fbTreeView.update(module, this.jSword, this.listbox.currentItem.getAttribute("type"));
	},

	loadCommentary: function() {
		var module = this.modules[this.listbox.currentIndex];
		if (typeof module == "undefined") return;
		fb.log("module is " + module);
		if (!this.jSword.isCommentaryModule(module)) return;
		var uri = "sword://" + module + "/" + fb.lastBibleURI;
		var mini = document.getElementById("firebible-mini");
		fb.log("loading commentary " + uri); 
		mini.loadURI(uri);
		fbTreeView.expandCommentaryView(true);
	},

	loadDictionary: function(key) {
		var char = key.substring(0, 1);
		var type;
		var prefs = fb.getJSword().fbPrefs;
		var defaultPref;
		if (char == "M") {
			type = 2;
			defaultPref = prefs.DEFAULT_GREEK_MORPH;
		} else if (char == "H") {
			type = 1;
			defaultPref = prefs.DEFAULT_HEBREW;
		} else {
			type = 0;
			defaultPref = prefs.DEFAULT_GREEK;
		}
		if (type < 2) {
			key = key.substring(1, 6);
			while (key.length < 5) {
				key = "0" + key;
			}
		} else {
			key = key.substr(2);
		}
		fb.log("key is " + key + ", type is: " + type);
		//check if currently selected module is of the required type
		var module = this.modules[this.listbox.currentIndex];
		var book;
		if (module && fb.getJSword().isDictionaryModule(module, type)) {
			book = module;
		} else {
			//get from preferences
			var bookName = prefs.getCharPref(defaultPref);
			if (bookName != null) {
				book = fb.getJSword().getModule(bookName);
				fb.log("od is " + book);				
			}
			if (book == null) { //bookName null or book no longer installed
				var dModules = fb.getJSword().getDictionaryModules(type);
				if (dModules.length > 0) {
					book = dModules[0].config;
				}
			}
		}
		if (book == null) {
			//prompt user and quit
			fb.promptService.alert(window, fb._("info.title"),
														fb._(type == 0 ? "greekDictNotFound.info"
																	 : (type == 1 ? "hebrewDictNotFound.info"
																			 		: "greekMorphNotFound.info")));
			return;
		}
		var bookIndex = this.bookIndexMap[book];
		this.listbox.ensureIndexIsVisible(bookIndex);
		this.listbox.selectedIndex = bookIndex;
		var idx = fb.getJSword().getDictionaryKeys(book).indexOf(key) - 1;
		if (idx < 0) {
			fb.promptService.alert(window, fb._("info.title"),
					fb._("keyNotFound.info", [key, book]));
			return;
		}
		fbTreeView.tree.view.selection.select(idx);
		fbTreeView.tree.boxObject.scrollToRow(idx);
	}
};

var fbTreeView = {

	init: function() {
		this.tree = document.getElementById("firebible-tree");
		this.mini = document.getElementById("firebible-mini");
		this.splitter = document.getElementById("firebible-splitter2");
		this.tree.view = this;
	},

	visibleData : [],

	update : function(module, jSword, type) {
		this.module = module;
		this.jSword = jSword;
		var removed = this.visibleData.length;
		this.visibleData = new Array();
		fb.log("book type is : " + type);
		if (type == "commentary") {
			fbItemList.loadCommentary();
			this.treeBox.rowCountChanged(0, -removed);
		} else {
			if (type == "dictionary" || type == "dailydevotional") {			
				var toc = this.jSword.getDictionaryKeys(module);
				this.addChildren(toc, 0, 0, removed, true, this.module, "");
			} else if (type == "genbook") {
				var toc = this.jSword.libSword.getGenBookTableOfContentsJSON(module);
				var tree = eval("(" + toc + ")");			
				this.addChildren(tree, 0, 0, removed, false, this.module, "");
			}
			this.expandCommentaryView(false);
			this.tree.view.selection.select(0);
			this.tree.treeBoxObject.scrollToRow(0);
		}
	},

	expandCommentaryView : function(expand) {
		this.splitter.setAttribute("state", (expand ? "collapsed" : "open"));
	},

	addChildren : function(pKey, index, level, removed, isList, module, keyString) {
//		var keys = this.jSword.utilKlass.getKeyChildren(pKey, module);
		var nChildren;
		var temp = new Array();
		if (isList) {
			nChildren = pKey.length - 1; // first entry seems to be unnecessary
			for (var i = 0; i < nChildren; i++) {
				var jItem = pKey[i + 1];
				temp[i] = [null, level, jItem, false, false];
			}
		} else {
			var i = 0;
			for (child in pKey) {
				var val = pKey[child];
				//value, level, text, isContainer, isContainerOpen, key
				temp[i++] = [val, level, child, (val != 1), false, keyString + "/" + child];
			}
			nChildren = i;
		}

		if (index == 0) {
			this.visibleData = temp;
		} else {
			for (var i = 0; i < nChildren; i++) {
				this.visibleData.splice(index + i, 0, temp[i]);
			}
		}
		this.treeBox.rowCountChanged(index, nChildren - removed);
		/*if (isList) {
			return keys[1];
		}*/
	},

	treeBox: null,
  	selection: null,

	  get rowCount()                     { return this.visibleData.length; },
	  setTree: function(treeBox)         { this.treeBox = treeBox; },
	  getCellText: function(idx, column) { return this.visibleData[idx][2]; },
	  isContainer: function(idx)         { return this.visibleData[idx][3]; },
	  isContainerOpen: function(idx)     { return this.visibleData[idx][4]; },
	  isContainerEmpty: function(idx)    { return false; },
	  isSeparator: function(idx)         { return false; },
	  isSorted: function()               { return false; },
	  isEditable: function(idx, column)  { return false; },

	  getParentIndex: function(idx) {
	    for (var t = idx - 1; t >= 0 ; t--) {
	      if (this.isContainer(t)) return t;
	    }
	    return -1;
	  },

	  getLevel: function(idx) {
	  	return this.visibleData[idx][1];
	  },

	  hasNextSibling: function(idx, after) {
	    var thisLevel = this.getLevel(idx);
	    for (var t = idx + 1; t < this.visibleData.length; t++) {
	      var nextLevel = this.getLevel(t)
	      if (nextLevel == thisLevel) return true;
	      else if (nextLevel < thisLevel) return false;
	    }
	  },

	  toggleOpenState: function(idx) {
	    var item = this.visibleData[idx];

	    if (!item[3]) return; //can't have children, shouldn't get here...

	    if (!item[4]) {
	      item[4] = true;
	      this.addChildren(item[0], idx + 1, item[1] + 1, 0, false, null, item[5]);
	    } else {
		  item[4] = false;
	      var thisLevel = this.getLevel(idx);
	      var deletecount = 0;
	      for (var t = idx + 1; t < this.visibleData.length; t++) {
	        if (this.getLevel(t) > thisLevel) deletecount++;
	        else break;
	      }
	      if (deletecount) {
	        this.visibleData.splice(idx + 1, deletecount);
	        this.treeBox.rowCountChanged(idx + 1, -deletecount);
	      }
	    }
	  },

	  keySelectionChanged: function() {
		if (this.tree.currentIndex == -1) return;
	  	var item = this.visibleData[this.tree.currentIndex];	  	
	  	var key;
	  	var uri = "sword://" + this.module;
	  	if (item && (item[0] == null)) {
	  		//not a tree key
	  		key = "/" + item[2];
	  	} else {
	  		key = item[5];
	  	}
	  	if (item[0] != null/*|| this.jSword.utilKlass.isMap(this.module)*/ || fbItemList.listbox.currentItem.getAttribute("type") == "dailydevotional") {
	  		fb.getGBrowser().loadURI(uri + key);
	  	} else {
	  		this.mini.loadURI(uri + key);
	  		// must set favicon URI here inspite of it being embedded in page,
	  		// it does not register when content is loaded in sidebar browser, so
	  		// history / bookmarks would appear without favicon
//	  		var favIconURI = this.jSword.utilKlass.getIconURIFor(this.module);
	  		var favIconURI = "resource://firebible/skin/book-d16.png";	  			
	  		var ios = Components.classes["@mozilla.org/network/io-service;1"].
	  		                            getService(Components.interfaces.nsIIOService);
	  		var iconURI = ios.newURI(favIconURI, null, null);
	  		var pageURI = ios.newURI(uri + key, null, null);
//	  		fb.getGBrowser().mFaviconService.setAndLoadFaviconForPage(pageURI,iconURI, true);
	  		PlacesUtils.favicons.setAndFetchFaviconForPage(pageURI, iconURI, true, null);	  																															
	  	}
	  },

	  getImageSrc: function(idx, column) {},
	  getProgressMode : function(idx,column) {},
	  getCellValue: function(idx, column) {},
	  cycleHeader: function(col, elem) {},
	  selectionChanged: function() {},
	  cycleCell: function(idx, column) {},
	  performAction: function(action) {},
	  performActionOnCell: function(action, index, column) {},
	  getRowProperties: function(idx, column, prop) {},
	  getCellProperties: function(idx, column, prop) {},
	  getColumnProperties: function(column, element, prop) {},

};