/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008-2012 Brian Fernandes <brian@thegoan.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/


var noun_type_verse = {
  label: "Bible verse",

  suggest: function(text, html) {
    var suggestions  = [];
    var books = fireBible_cmd.getFireBible(false).suggestBooks(text);
    if (books) {
      books = fireBible_cmd.sortSuggestions(books);
      for (var i = 0; i < books.length; i++) {
        suggestions.push(CmdUtils.makeSugg(books[i][0], books[i][0], null, books[i][3]));
      }
    }
    return suggestions;
  },

  noExternalCalls: true
};

var noun_type_book = {
    label: "Book",

    suggest: function (text, html) {
      var suggestions = [];
      var bibles = fireBible_cmd.getFireBible(false).suggestBibles(text);
      if (bibles) {
    	bibles = fireBible_cmd.sortSuggestions(bibles);
        for (var i = 0; i < bibles.length; i++) {
          suggestions.push(CmdUtils.makeSugg(bibles[i][0]));
        }
      }
      return suggestions;
    },

	noExternalCalls: true
};


var noun_type_destination =  {
	destinations: ["firefox", "clipboard"],
	label: "Destination",

	suggest: function(text, html) {
		var suggestions = [];
		for (var i = 0; i < this.destinations.length; i++) {
			if (this.destinations[i].match(text, "i")) {
				suggestions.push(CmdUtils.makeSugg(this.destinations[i]));
			}
		}
		return suggestions;
	},

	noExternalCalls: true

};


var fireBible_cmd = {
  name: ["sword", "lookup bible"],
  homepage: "http://thegoan.com/firebible",
  author: { name: "Brian Fernandes", email: "brian@thegoan.com"},
  license: "GPL",
  icon: "chrome://firebible/skin/bible16.png",
  description: "Lookup Bible verses from FireBible",
  arguments: [{role: "object", nountype: noun_type_verse, label: "verse"},
              {role: "source", nountype: noun_type_book, label: "bible"},
              {role: "goal", nountype: noun_type_destination, label: "destination"}],
  Cc: Components.classes,
  Ci: Components.interfaces,
  
  preview: function(pblock, args) {
    var info = this.getURI(args.object, args);
    var url = info[0];
    var verse = info[1];
    if (!url) {
    	if (args.source.text.length == 0 && args.goal.text.length == 0) {
    		pblock.innerHTML = "Specify verse reference";
    	} else {
    		pblock.innerHTML = "<span style = 'color: #f00;'>Please begin your query with a verse reference</span>";
    	}
    	return;
    }
    var fireBible = this.getFireBible(true);
    var bible = info[2];
    if (!bible) {
    	//bible was not specified, get the default bible
    	bible = fireBible.getJSword().getDefaultBible();
    }
    var cacheInvalid = (this.pVerse != verse || this.pBible != bible);
    if (cacheInvalid) {
    	if (!fireBible.getJSword().utilsInitialized) { //FIXME unlikely to happen?
    		pblock.innerHTML = "Please wait while FireBible is initialized";
    		return;
    	}
    	this.pBible = bible;
    	this.pVerse = verse;
    	this.html = fireBible.load(url, 2)[0];
    }    
//    var newChild = this.parseHTML(pblock, this.html, true, this.getBaseURI(), false);
//    pblock.replaceChild(newChild, pblock.lastChild);
    pblock.innerHTML = this.html;
  },
  
  parseHTML: function(doc, html, allowStyle, baseURI, isXML) {
	  const PARSER_UTILS = "@mozilla.org/parserutils;1";
	 
	  //User the newer nsIParserUtils on versions that support it.
	  if (PARSER_UTILS in Components.classes) {
		  let parser = Components.classes[PARSER_UTILS]
	                               .getService(Ci.nsIParserUtils);
		  if ("parseFragment" in parser)
	      return parser.parseFragment(html, allowStyle ? parser.SanitizerAllowStyle : 0,
	                                        isXML, baseURI, doc);
	  }
	 
	  return Components.classes["@mozilla.org/feed-unescapehtml;1"]
	                     .getService(Components.interfaces.nsIScriptableUnescapeHTML)
	                     .parseFragment(html, !!isXML, baseURI, doc);
  },
	
  getBaseURI: function() {
	  if (this.baseURI == null) {
		  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
			                  .getService(Components.interfaces.nsIIOService);
		  this.baseURI = ioService.newURI("bible://", "UTF-8", null);
	  }
	  return this.baseURI;
  },

  execute: function(args) {
	  var url = this.getURI(args.object, args)[0];
	  if (!url) {
		  displayMessage("Please begin your query with a verse reference");
		  return;
	  }
	  var open = "bible://" + escape(url);
	  var fb = context.chromeWindow.fireBible;
	  var to = args.goal.text;
	  if (!to || to.length == 0) {
		  var textEnabled = false;
		  if (context.focusedWindow.document.designMode == "on") {
			  textEnabled = true;
		  } else {
			  var node = context.focusedElement;
			  if (node && node.type == "textarea") {
				  textEnabled = true;
			  }
		  }
		  if (!textEnabled) {
			  var prefs = fb.getJSword().fbPrefs;
			  var def = prefs.getIntPref(prefs.DEFAULT_ACTION);
			  if (def < 0 || def > 1) def = 0;
			  to = noun_type_destination.destinations[def];
		  } else {
			  to = "text";
		  }
	  }
	  if (noun_type_destination.destinations[0].match(to, "i")) { //firefox
		  Utils.openUrlInBrowser(open);
	  } else {
		  var content = fb.load(url, 1)[0];
		  var html = this.getString(content);
		  var text = this.getTextFromHTML(content.replace(/<sup class='verse'>(.*?)<\/sup>/gi, "($1) "));
		  if (noun_type_destination.destinations[1].match(to, "i")) { //clipboard
			  this.copyToClipboard(html, text);
		  } else if ("text".match(to, "i")) {
			  var options = new Object();
			  options.text = text.toString();
			  CmdUtils.setSelection(html, options);
		  }
	  }
  },

  getURI: function(input, args) {
	  if (!input.text || input.text.length == 0) return [null, false];
	  var bibleName = args.source.text;
	  var nameSpecified = bibleName.length > 0;
	  var uri = nameSpecified ? (bibleName + "/" + input.text) : input.text;
	  return [uri, input.text, (nameSpecified ? bibleName : null)];
  },

  copyToClipboard: function(html, text) {
	  var trans = Cc["@mozilla.org/widget/transferable;1"].
	                createInstance(Ci.nsITransferable);

	  if ('init' in trans) {
		  trans.init(null);
	  }
	  //add text/html data to clipboard
	  trans.addDataFlavor("text/html");
	  trans.setTransferData("text/html", html, html.data.length);

	  //add text/unicode data to clipboard
	  trans.addDataFlavor("text/unicode");
	  trans.setTransferData("text/unicode", text, text.data.length * 2);

	  var clipboard = Cc["@mozilla.org/widget/clipboard;1"].
	                    getService(Ci.nsIClipboard);

	  clipboard.setData(trans, null, Ci.nsIClipboard.kGlobalClipboard);
  },

  getString: function(str) {
	  var string = Cc["@mozilla.org/supports-string;1"].
	  								createInstance(Ci.nsISupportsString);
	  string.data = str;
	  return string;
  },

  getTextFromHTML: function(str) {
	  var string = this.getString(str);
	  var converter = Cc["@mozilla.org/widget/htmlformatconverter;1"].
											createInstance(Ci.nsIFormatConverter);
	  var text = { value: null };
	  converter.convert("text/html", string, str.length, "text/unicode", text, {});
	  return text.value.QueryInterface(Ci.nsISupportsString);
  },

  getFireBible: function(forceInit) {
	  var chromeWindow = Cc["@mozilla.org/appshell/window-mediator;1"].
	                       getService(Ci.nsIWindowMediator).
	                       getMostRecentWindow("navigator:browser");
	  var fireBible = chromeWindow.fireBible;
	  var jsword = fireBible.getJSword();
	  if (!jsword.utilsInitialized && !jsword.initializing) {
		  var init = forceInit ||
		  					 jsword.fbPrefs.getBoolPref(jsword.fbPrefs.ENABLE_UBIQUITY_INIT);
		  if (!init) {//has the user typed "sword" or "lookup bible"?
			  try {
				  var selectedCmd = chromeWindow.gUbiquity.textBox.value;
				  init = (selectedCmd.match("^sword\\s", "i") ||
				  				selectedCmd.match("^lookup\\s*bible\\s", "i"));
			  } catch (e) {
				  CmdUtils.log("Error getting textBox content");
			  }
		  }
		  if (init) fireBible.initFromProtocol(false);
	  }
	  return fireBible;
  },

  sortSuggestions: function(suggestions) {
	  return suggestions.sort(function(a, b) {
		  if (a[2] ==  b[2]) {
			  // 3rd element of the array is a boolean indicating whether the
			  // match was at the start of the text
			  return a[0] > b[0];
		  } else {
			  return !a[2];
		  }
	  });
  }

};

CmdUtils.CreateCommand(fireBible_cmd);
