/*  This file is part of xulSword.

    Copyright 2009 John Austin (gpl.programs.info@gmail.com)

    xulSword is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    xulSword is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with xulSword.  If not, see <http://www.gnu.org/licenses/>.
*/

// IMPORTANT INFO ABOUT THIS FILE:
// The functions in common0.js may be used even before the XS_window has 
// loaded. But NO FUNCTION IN THIS FILE may use LibSword or call another 
// function which uses LibSword. Otherwise there will be unhandled excep-
// tions. This file also assigns global constants and it should be loaded
// into every scope which uses any common Javascript code. Naturally, 
// functions in this file must properly handle any exceptions which may
// arise from unset XS_window globals.

function XULSword(cLocation) {
	this.cLocation = cLocation;
	
	// What's our operating system?
	this.OPSYS = "Unknown OS";
	if (navigator.appVersion.indexOf("Win")!=-1) this.OPSYS="WINNT";
	else if (navigator.appVersion.indexOf("Mac")!=-1) this.OPSYS="MacOS";
	else if (navigator.appVersion.indexOf("Linux")!=-1) this.OPSYS="Linux";
	else if (navigator.appVersion.indexOf("X11")!=-1) this.OPSYS="Linux";
	
	this.BIN = { WINNT:"dll", Linux:"so" };
}

//var IsExtension;

// BOOKMARK CONSTANTS:
// NOTE: THE LIST BELOW CANNOT BE CHANGED WITHOUT BREAKING COMPATIBILITY WITH EARLIER EXPORTED BOOKMARKS
// "TYPE" - "BookmarkSeparator", "Folder", "EmptyBookmark", or "Bookmark".
// "NAME" - Default name is generated based on TYPE and MODULE.
// "NOTE"  - User note.
// "BOOK" - Bible book for MODULEs using versekey.
// "CHAPTER" - 
//    Bible chapter for MODULEs using versekey, 
//    key for dictionary MODULEs, 
//    module:key for GENBOOK MODULEs
// "VERSE" - 
//    Bible verse for MODULEs using versekey.
//    paragraph for non-versekey MODULEs
// "LASTVERSE" - Bible lastVerse for MODULEs using versekey.
// "MODULE" - Module name
// "LOCATION" - Location in form bookShortName.chap.verse[.lastverse] ALWAYS WITH EASTERN VERSIFICATION
// "BMTEXT" - Bookmarked text sample
// "ICON" - URL to Texts.png, Comms.png, Dicts.png, Genbks.png. Or TextsWithNote.png etc. (add WithNote)
// "CREATIONDATE" - Creation month, day, year, localized to operating system locale.
// "VISITEDDATE" - Last visited month, day, year, localized to operating system locale. (When is this updated?)
//THE FOLLOWING WERE INTRODUCED IN v2.12. EARIER BOOKMARKS WILL NOT HAVE THESE PARAMETERS
// "NAMELOCALE" - Program locale when NAME was set.
// "NOTELOCALE" - Program locale when note was set.



XULSword.prototype.IsExtension = true;
XULSword.prototype.APPLICATIONID = "firebible@thegoan.com";
//XULSword.prototype.APPLICATIONID = "testExtension";


/************************************************************************
 * Declare/Define Common Global Constants
 ***********************************************************************/
XULSword.prototype.ORIGINAL = "ORIG";        //Value doen't really matter, just a const
XULSword.prototype.BIBLE = "Biblical Texts";
XULSword.prototype.DICTIONARY = "Lexicons / Dictionaries";
XULSword.prototype.COMMENTARY = "Commentaries";
XULSword.prototype.GENBOOK = "Generic Books";
XULSword.prototype.NOTFOUND = "Not Found";
XULSword.prototype.DEFAULTLOCALE = "en-US";
XULSword.prototype.PMSTD="centerscreen, dependent";
XULSword.prototype.PMSPLASH="alwaysRaised,centerscreen";
XULSword.prototype.PMMODAL="alwaysRaised,centerscreen,modal";
XULSword.prototype.PMNORMAL=0;
XULSword.prototype.PMSTOP=1;
XULSword.prototype.FPERM = 511; //0777;
XULSword.prototype.DPERM = 511; //0777;
XULSword.prototype.MANIFEST_EXT=".manifest";
XULSword.prototype.CONF_EXT=".conf";
XULSword.prototype.EXTENSION_EXT=".xpi";
XULSword.prototype.MODSD="mods.d";
XULSword.prototype.MODS="modules";
XULSword.prototype.LOCALE="locale";
XULSword.prototype.CHROME="chrome";
XULSword.prototype.FONTS="fonts";
XULSword.prototype.AUDIO="audio";
XULSword.prototype.BOOKMARKS="bookmarks"
XULSword.prototype.VIDEO="video";

XULSword.prototype.SupportedModuleTypes = {
  Texts: XULSword.prototype.BIBLE,
  Comms: XULSword.prototype.COMMENTARY,
  Dicts: XULSword.prototype.DICTIONARY,
  Genbks: XULSword.prototype.GENBOOK
};


XULSword.prototype.Config = {
  direction:        { modConf:"Direction", localeConf:"Direction", CSS:"direction" },
  fontFamily:       { modConf:"Font", localeConf:"Font", CSS:"font-family" },
  fontSizeAdjust:   { modConf:"FontSizeAdjust", localeConf:"FontSizeAdjust", CSS:"font-size-adjust" },
  lineHeight:       { modConf:"LineHeight", localeConf:"LineHeight", CSS:"line-height" },
  fontSize:         { modConf:"FontSize", localeConf:"FontSize", CSS:"font-size" },
  color:            { modConf:"FontColor", localeConf:"FontColor", CSS:"color" },
  background:       { modConf:"FontBackground", localeConf:"FontBackground", CSS:"background" },
  AssociatedModules:{ modConf:null, localeConf:"DefaultModule", CSS:null },
  AssociatedLocale: { modConf:null, localeConf:null, CSS:null },
  StyleRule:        { modConf:null, localeConf:null, CSS:null },
  TreeStyleRule:    { modConf:null, localeConf:null, CSS:null }
};

XULSword.prototype.prefs = Components.classes["@mozilla.org/preferences-service;1"]
.getService(Components.interfaces.nsIPrefService) 
.getBranch("extensions.firebible.");

XULSword.prototype.escapeRE = function(text) {
  const ESCAPE_RE= new RegExp(/([\-\[\]\(\)\{\}\+\*\.\:\^\$\?\|\\])/g);
  return text.replace(ESCAPE_RE, "\\$1");
}

XULSword.prototype.jsdump = function(str) {
  window.dump(str + "\n");
  try {Components.classes['@mozilla.org/consoleservice;1']
            .getService(Components.interfaces.nsIConsoleService)
            .logStringMessage(str);}
	catch (er) {}
}

function jsdump(str) {
  window.dump(str + "\n");
  try {Components.classes['@mozilla.org/consoleservice;1']
            .getService(Components.interfaces.nsIConsoleService)
            .logStringMessage(str);}
	catch (er) {}
}


//creates only allowable file types
XULSword.prototype.createSafeFile = function(nsIFile, perm, createUnique) {
  if (!nsIFile || !nsIFile.QueryInterface(Components.interfaces.nsILocalFile)) return 0;
  
  // only create a file if it has one of these file extensions
  if (!(/\.(txt|xsb|rdf|conf|xpi)$/i).test(nsIFile.leafName)) {
    jsdump("ERROR: createSafeFile \"" + nsIFile.path + "\"");
    return 0;
  }
  
  if (createUnique) nsIFile.createUnique(nsIFile.NORMAL_FILE_TYPE, perm);
  else nsIFile.create(nsIFile.NORMAL_FILE_TYPE, perm);
  
  return 1;
}

XULSword.prototype.readParamFromConf = function(nsIFileConf, param) {
	  nsIFileConf = nsIFileConf.QueryInterface(Components.interfaces.nsILocalFile);
	  if (!nsIFileConf) return "";
	  if (nsIFileConf.leafName.search(".conf", "i") == -1) return null;
	  
	  var filedata = xulSword.readFile(nsIFileConf);
	  
	  if (param == "ModuleName") {
	    var prm = new RegExp("^\\s*\\[(.*)\\]", "m");
	    var retval = filedata.match(prm);
	    if (retval) retval = retval[1];
	  }
	  else {
	    prm = new RegExp("^\\s*" + this.escapeRE(param) + "\\s*=\\s*(.*?)\\s*?[\\r\\n]", "im");
	    retval = filedata.match(prm);
	    if (retval) retval = retval[1];
	  }
	  return retval;
	}

XULSword.prototype.removeFile = function(file, recurse) {
	  if (!recurse) recurse = false;
	  if (file) file = file.QueryInterface(Components.interfaces.nsILocalFile);
	  if (!file) return false;
	  if (!file.exists()) return true;
	  try {file.remove(recurse);}
	  catch (er) {jsdump("Failed to remove " + (file.isDirectory() ? "directory":"file") + " \"" + file.path + "\", recurse=" + recurse + ". " + er); return false;}
	  return true;
	}


XULSword.prototype.getModuleConfigDefaultCSS = function() {
	var moduleConfigDefaultCSS = {
		fontFamily:this.getPrefOrCreate("user.fontFamily.default", "Char", "'Arial'"),
		direction: this.getPrefOrCreate("user.direction.default", "Char", "ltr"),
		fontSizeAdjust: this.getPrefOrCreate("user.fontSizeAdjust.default", "Char", "none"),
		lineHeight: this.getPrefOrCreate("user.lineHeight.default", "Char", "1.6em"),
		fontSize: this.getPrefOrCreate("user.fontSize.default", "Char", "1em"),
		color: this.getPrefOrCreate("user.color.default", "Char", "#202020"),
		background: this.getPrefOrCreate("user.background.default", "Char", "unspecified")
	};
	
	return moduleConfigDefaultCSS;
}

XULSword.prototype.createStyleRule = function(selector, config) {
  var rule = selector + " {";
  for (var p in this.Config) {
    if (!this.Config[p].CSS || config[p] == "unspecified") continue;
    rule += this.Config[p].CSS + ":" + config[p] + "; ";
  }
  rule += "}";
//jsdump(rule);
  return rule;
}

XULSword.prototype.getPrefOrCreate = function(prefName, prefType, defaultValue) {
	  var prefs = this.prefs;
	  var prefVal;
	  try {
	    switch (prefType) {
	    case "Char":
	      prefVal = prefs.getCharPref(prefName);
	      break;
	    case "Bool":
	      prefVal = prefs.getBoolPref(prefName);
	      break;
	    case "Int":
	      prefVal = prefs.getIntPref(prefName);
	      break;
	    case "Unicode":
	      prefVal = getUnicodePref(prefName);
	      break;
	    default:
	      this.jsdump("Warning!: Could not handle pref type: " + prefType + "\n");
	      return null;
	    }
	  }
	  catch (er) {
	    prefVal = defaultValue;
	    switch (prefType) {
	    case "Char":
	      prefs.setCharPref(prefName,prefVal);
	      break;
	    case "Bool":
	      prefs.setBoolPref(prefName,prefVal);
	      break;
	    case "Int":
	      prefs.setIntPref(prefName,prefVal);
	      break;
	    case "Unicode":
	      setUnicodePref(prefName,prefVal);
	      break;
	    }
	  }

	  return prefVal;
	}

//Convert file path separators into native platform separators 
XULSword.prototype.lpath = function(path) {
  if (this.OPSYS == "WINNT") {path = path.replace(/\//g, "\\");}
  else if (this.OPSYS == "Linux") {path = path.replace(/\\/g, "/");}
  return path;
}

//in addition to the XULRunner special directories, the following are also recognized
//xsResD       = resource directory
//xsFonts      = user fonts directory
//xsAudio      = user audio directory
//xsBookmarks  = user bookmarks directory
//xsModsUser   = user SWORD module directory (contains mods.d & modules)
//xsModsCommon = shared SWORD module directory (contains mods.d & modules)
//xsExtension  = profile extensions directory
//xsLocale     = libsword locale directory
//xsDefaults   = xulsword's defaults directory
//xsProgram    = xulsword's program files root directory

XULSword.prototype.createAppDirectories = function() {
  var checkdir = this.getSpecialDirectory("xsResD");   if (!checkdir.exists()) checkdir.create(checkdir.DIRECTORY_TYPE, this.DPERM);
  checkdir = this.getSpecialDirectory("xsModsUser");   if (!checkdir.exists()) checkdir.create(checkdir.DIRECTORY_TYPE, this.DPERM);
  var checkdi2 = checkdir.clone();
  checkdi2.append(this.MODSD);                         if (!checkdi2.exists()) checkdi2.create(checkdi2.DIRECTORY_TYPE, this.DPERM);
  checkdir.append(this.MODS);                          if (!checkdir.exists()) checkdir.create(checkdir.DIRECTORY_TYPE, this.DPERM);
};

XULSword.prototype.getSpecialDirectory = function(name) {
var directoryService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
if (name.substr(0,2) == "xs") {

	
// var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
// file.initWithPath(extensionPath);  
 var profile = directoryService.get("ProfD", Components.interfaces.nsILocalFile);
 
 var resources = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
 var path = profile.path;
 if (this.IsExtension) path += "/xulsword_resources";
 else path = path.replace(new RegExp(this.escapeRE(profile.leafName) + "$"), "resources");
 resources.initWithPath(this.lpath(path));
 
 var retval = resources.clone().QueryInterface(Components.interfaces.nsILocalFile);
 
 switch(name) {
 case "xsFonts":
   retval.append(XULSword.prototype.FONTS);
   break;
 case "xsAudio":
   retval.append(XULSword.prototype.AUDIO);
   break;
 case "xsBookmarks":
   retval.append(XULSword.prototype.BOOKMARKS);
   break;
 case "xsVideo":
   retval.append(XULSword.prototype.VIDEO);
   break;
 case "xsLocale":
   retval.append("locales.d");
   break;
 case "xsExtension":
   retval = profile.clone();
   retval.append("extensions");
   break;
 case "xsResD":
 case "xsModsUser":
   // already correct...
   break;
 case "xsExtResource":
   if (this.IsExtension) {
     retval = profile.clone();
     retval.append("extensions");
     retval.append(this.APPLICATIONID);
     retval.append("resources");
   }
   // else return regular resources directory
   break;
 case "xsDefaults":
   if (this.IsExtension) {
     retval = profile.clone();
     retval.append("extensions");
     retval.append(this.APPLICATIONID);
     retval.append("defaults");
   }
   else {
     retval = directoryService.get("DefRt", Components.interfaces.nsILocalFile);
   }
   break;
 case "xsProgram":
   if (this.IsExtension) {
     retval = profile.clone();
     retval.append("extensions");
     retval.append(this.APPLICATIONID);
   }
   else {
     retval = directoryService.get("CurProcD", Components.interfaces.nsILocalFile);
   }
   break;
 case "xsModsCommon":
   switch (this.OPSYS) {
   case "WINNT":
     var userAppPath = Components.classes["@mozilla.org/process/environment;1"].
         getService(Components.interfaces.nsIEnvironment).get("APPDATA");
     userAppPath += "/Sword";
     break;
   case "Linux":
     var userAppPath = Components.classes["@mozilla.org/process/environment;1"].
         getService(Components.interfaces.nsIEnvironment).get("HOME");
     userAppPath += "/.sword";
     break;
   }
   retval.initWithPath(lpath(userAppPath));
   break;
 }
}
else {
 retval = directoryService.get(name, Components.interfaces.nsILocalFile);
}
return retval;
};

//returns data from file. Does little checking!
XULSword.prototype.readFile = function(nsIFile) {
  if (!nsIFile || !nsIFile.exists()) return "";
  var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
  fstream.init(nsIFile, -1, 0, 0);
  var charset = "UTF-8";
  const replacementChar = Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER;
  var is = Components.classes["@mozilla.org/intl/converter-input-stream;1"].createInstance(Components.interfaces.nsIConverterInputStream);
  is.init(fstream, charset, 1024, replacementChar);
  var filedata = "";
  var str = {};
  while (is.readString(4096, str) != 0) {filedata = filedata + str.value;}
  fstream.close();
  is.close();
  return filedata;
};


XULSword.prototype.closeWindowXS = function(aWindow) {
	  if (typeof(AllWindows) == "object" && 
	      AllWindows instanceof Array && 
	      AllWindows.length) {
	    var i = AllWindows.indexOf(aWindow);
	    if (i != -1) AllWindows.splice(i, 1);
	  }
	  aWindow.close();
};
