XULSword.prototype.getModuleLongType = function(aModule) {
  if (aModule == this.ORIGINAL) return this.BIBLE;
  var typeRE = new RegExp("(^|<nx>)\\s*" + this.escapeRE(aModule) + "\\s*;\\s*(.*?)\\s*(<nx>|$)");
  var type = LibSword.getModuleList().match(typeRE);
  if (type) type = type[2];
  return type;
};


XULSword.prototype.getModuleConfig = function(mod) {
  var moduleConfig = {};
  
  var moduleConfigDefaultCSS = this.getModuleConfigDefaultCSS();
  // All versionconfig members should have a valid value, and it must not be null.
  // Read values from module's .conf file
  for (var p in this.Config) {
    if (!this.Config[p].modConf) continue;
    if (mod != "LTR_DEFAULT") {
			var val = LibSword.getModuleInformation(mod, this.Config[p].modConf);
		}
		else val = "";
    if ((/^\s*$/).test(val)) val = this.NOTFOUND;

    if (val == this.NOTFOUND && this.Config[p].CSS && moduleConfigDefaultCSS[p]) {
      val = moduleConfigDefaultCSS[p];
    }
  
    // allow user to overwrite module defaults
    try {
			var userVal = this.prefs.getCharPref("user." + p + "." + mod);
			if (userVal) val = userVal;
		} catch (er) {}
    
    moduleConfig[p] = val;
  }
  
  var LocaleConfigs = XULSword.prototype.LocaleConfigs;
    
  // Assign associated locale and modules
  if (mod != "LTR_DEFAULT") {
		moduleConfig["AssociatedLocale"] = this.getLocaleOfModule(mod);
		if (!moduleConfig["AssociatedLocale"]) moduleConfig["AssociatedLocale"] = this.NOTFOUND;
		
		if (moduleConfig["AssociatedLocale"] != this.NOTFOUND && LocaleConfigs.hasOwnProperty(moduleConfig["AssociatedLocale"]))
				moduleConfig["AssociatedModules"] = LocaleConfigs[moduleConfig["AssociatedLocale"]].AssociatedModules;
		else moduleConfig["AssociatedModules"] = this.NOTFOUND;
	}
	else {
		moduleConfig["AssociatedLocale"] = this.DEFAULTLOCALE;
		moduleConfig["AssociatedModules"] = this.NOTFOUND;
	}
	
  // Normalize direction value
  moduleConfig.direction = (moduleConfig.direction.search("RtoL", "i") != -1 ? "rtl":"ltr");
  
	// if fontFamily specifies a font URL, rather than a fontFamily, then create a
	// @font-face CSS entry and use it for this module.
	var url = moduleConfig.fontFamily.match(/(\w+\:\/\/[^"'\)]+)\s*$/);
	if (url) {
		var fam = "_" + url[1].replace(/[^\w\d]/g, "_");
		FontFaceConfigs[fam] = url[1];
		moduleConfig.fontFamily = fam;
		//this.jsdump("INFO: " + mod + " is using font-face " + fam + ": " + url[1]);
	}

  // Insure there are single quotes around font names
  moduleConfig.fontFamily = moduleConfig.fontFamily.replace(/\"/g, "'");
  if (moduleConfig.fontFamily != this.NOTFOUND && !(/'.*'/).test(moduleConfig.fontFamily)) 
      moduleConfig.fontFamily = "'" + moduleConfig.fontFamily + "'";

  // Save the CSS style rules for this module, which can be appended to CSS stylesheets
  moduleConfig.StyleRule = this.createStyleRule(".cs-" + mod, moduleConfig);
  // "m" is prepended to mod because some module names begin with a digit,
  // which would otherwise result in an invalid selector
  moduleConfig.TreeStyleRule = this.createStyleRule("treechildren::-moz-tree-cell-text(m" + mod + ")", moduleConfig);
  
  return moduleConfig;
}

XULSword.prototype.getLocaleOfModule = function(module) {
	  var myLocale=null;
	  
	  var LocaleConfigs = XULSword.prototype.LocaleConfigs;
	  
	  for (var lc in LocaleConfigs) {
	    var regex = new RegExp("(^|\s|,)+" + module + "(,|\s|$)+");
	    if (LocaleConfigs[lc].AssociatedModules.match(regex)) myLocale = lc;
	  }
	  
	  if (myLocale) return myLocale;
	  
	  for (lc in LocaleConfigs) {
	    var lcs, ms;
	    try {
	      lcs = lc.toLowerCase();
	      ms = LibSword.getModuleInformation(module, "Lang").toLowerCase();
	    }
	    catch(er) {lcs=null; ms==null;}
	    
	    if (ms && ms == lcs) {myLocale = lc; break;}
	    if (ms && lcs && ms.replace(/-.*$/, "") == lcs.replace(/-.*$/, "")) {
	      myLocale = lc;
	      if (myLocale == getLocale()) break;
	    }
	  }
	  
	  return myLocale;
}
