/*  This file is part of xulSword.

    Copyright 2009 John Austin (gpl.programs.info@gmail.com)

    xulSword is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    xulSword is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with xulSword.  If not, see <http://www.gnu.org/licenses/>.
*/

////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
// This window creates xulsword profile directories, installs modules, 
// and then either quits (if xsInstallOnly is set) or opens the splash window
// and main xulsword window (the usual occurence). The main xulsword window 
// closes this hidden window during its init sequence, which in turn closes the
// visible splash window.
Components.utils.import("resource://gre/modules/Downloads.jsm");
Components.utils.import("resource://gre/modules/osfile.jsm")
Components.utils.import("resource://gre/modules/Task.jsm");  
  
var SWLibrary = {};

XULSword.prototype.startxulsword = function () {  
  /*// Check that we have libxulsword installed
  if (!BIN.hasOwnProperty(OPSYS)) {
    jsdump("ERROR: This platform is currently not supported. Exiting...");
    closeWindowXS(window);
    return;
  }*/
  this.createAppDirectories();	
	
  var xulABI = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime);
  try {this.xulABI = xulABI.XPCOMABI;}
  catch (er) {this.xulABI = "unknown";}
  SWLibrary.status = null;
  SWLibrary.libName = "libxulsword-" + this.prefs.getCharPref("LibxulswordVersion") + "-" + this.OPSYS + "_" + this.xulABI + "." + this.BIN[this.OPSYS];
  SWLibrary.file = this.getSpecialDirectory("xsProgram");
  SWLibrary.file.append(SWLibrary.libName);
  if (SWLibrary.file.exists()) {
    return true;
  }
  // If not installed, check whether it can be downloaded
  /*try {SWLibrary.url = prefs.getCharPref("LibSwordURL") + "/" + SWLibrary.libName + ".zip";}
  catch (er) {
    jsdump("ERROR: No libxulsword and no LibSwordURL. Exiting...");
    closeWindowXS(window);
    return;
  }*/
  
  // Prompt for libxulsword download permission
  var result={};
  SWLibrary.opener = (window.opener ? window.opener:window);
  if (!fireBible.promptService.confirm(SWLibrary.opener, "LibSword Download Required", "To use FireBible, a binary executable library must be downloaded.\n\nThis library has NOT been screened by Mozilla.\n\nDo you wish to download this library?")) {
	  return false;
  };
  
  SWLibrary.opener.openDialog("chrome://firebible/content/xulsword/workProgress/workProgress.xul", "work-progress", this.PMMODAL,
	this.PMSTOP,
    "Downloading Library",
    xulSword,
    "Downloading", 
    true
  );  
  
  return (SWLibrary.status == null);
}


XULSword.prototype.downloadFTPListing = function(progressMeter) {
  // Request the latest compatible libxulsword version available (by reading the FTP directory listing)
  // This only works if LibSwordURL protocol is FTP, otherwise LibxulswordVersion will be requested.
  SWLibrary.progressMeter = progressMeter;
  SWLibrary.listingFile = this.getSpecialDirectory("TmpD");
  SWLibrary.listingFile.append("libxulswordListing.txt");
  if (SWLibrary.listingFile.exists()) SWLibrary.listingFile.remove(false);
  
  Task.spawn(function() {
		  let download = yield Downloads.createDownload(
				  {
					  source: xulSword.prefs.getCharPref("LibSwordURL"),
					  target: SWLibrary.listingFile
				  });
		  
		  SWLibrary.download = download;
//		  yield Downloads.fetch(xulSword.prefs.getCharPref("LibSwordURL"), SWLibrary.listingFile);	     
//	      xulSword.downloadLibxulsword();
		  
		  yield download.start();	  
	  return;
  }).then(function() {xulSword.downloadLibxulsword();}, xulSword.downloadError);
	
};



XULSword.prototype.downloadLibxulsword = function() {
	// see if a newer compatible library version is available
	// If both primary and secondary version numbers match those of LibxulswordVersion, 
	// then the lib is compatible. If a tertiary version number is also present, choose
	// the lib with the highest tertiary.
	 var data = xulSword.readFile(SWLibrary.listingFile);
	 var files = data.match(/201\: \"(.*?)\" (\d+) .*? FILE\s*$/gm);
	 var libvers = xulSword.prefs.getCharPref("LibxulswordVersion");
	 var comparator = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
	 var compatible = new RegExp(
	   xulSword.escapeRE("libxulsword-") + 
	   "(" + xulSword.escapeRE(libvers.split(".")[0] + "." + libvers.split(".")[1]) + "[^\\-]*)" + 
	   xulSword.escapeRE("-" + this.OPSYS + "_" + xulSword.xulABI + "." + this.BIN[this.OPSYS])
	 ); // is only compatible if major and secondary versions are identical, in addition to platform etc.
	 
	 for (var i=0; files && i<files.length; i++) {
	   var file = files[i].match(/201\: \"(.*?)\" (\d+) .*? FILE\s*$/)[1];
	   var vers = file.match(compatible);
	   if (vers && comparator.compare(vers[1], libvers) > 0) libvers = vers[1];
	 }
	
	 SWLibrary.libName = "libxulsword-" + libvers + "-" + this.OPSYS + "_" + xulSword.xulABI + "." + this.BIN[this.OPSYS];
	 SWLibrary.file = xulSword.getSpecialDirectory("xsProgram");
	 SWLibrary.file.append(SWLibrary.libName);
	 SWLibrary.url = xulSword.prefs.getCharPref("LibSwordURL") + "/" + SWLibrary.libName + ".zip";
	//     alert("need to download" + SWLibrary.url);
	     SWLibrary.progressMeter.setLibraryName(SWLibrary.libName);
	  // Download libxulsword (as a zip file for quicker download)
  SWLibrary.zipFile = this.getSpecialDirectory("TmpD");
  SWLibrary.zipFile.append(SWLibrary.libName + ".zip");
  if (SWLibrary.zipFile.exists()) SWLibrary.zipFile.remove(false);  

  Task.spawn(function () {
	  let download = yield Downloads.createDownload(
						  {
							  source: SWLibrary.url,
							  target: SWLibrary.zipFile						  	
						  });
	  
	  SWLibrary.download = download;
	  
	  download.onchange = function() {
		  SWLibrary.progressMeter.Progress.setAttribute("value", download.progress); 
	  };
	  
	  yield download.start();
	  
  }).then(function() {xulSword.downloadCompleted();}, xulSword.downloadError);
}

XULSword.prototype.downloadCompleted = function() {
	 this.closeWindowXS(SWLibrary.progressMeter);
	 SWLibrary.file.create(SWLibrary.file.NORMAL_FILE_TYPE, this.FPERM);
     
     var zReader = Components.classes["@mozilla.org/libjar/zip-reader;1"].createInstance(Components.interfaces.nsIZipReader);
     zReader.open(SWLibrary.zipFile);
     try {zReader.extract(SWLibrary.zipFile.leafName.replace(".zip",""), SWLibrary.file);}
     catch (er) {
       SWLibrary.status = "nsIZipReader \"" + SWLibrary.zipFile.leafName + "\".";
     }
     zReader.close(SWLibrary.zipFile);
     SWLibrary.file.permissions = this.FPERM; // zReader sets permissions!
     if (SWLibrary.zipFile.exists()) SWLibrary.zipFile.remove(false);
     this.prefs.setCharPref("LibxulswordVersion", SWLibrary.file.leafName.match(/libxulsword\-([^\-]*)\-/)[1]);
};

XULSword.prototype.downloadError = function(ex) {
	SWLibrary.status = "Download error";
	if (!SWLibrary.download.canceled) {
		fireBible.promptService.alert(SWLibrary.opener, "Error", "Error downloading LibSword. Please ensure you are online and try again.");
	}	
	fireBible.jsword.log(ex);
	xulSword.closeWindowXS(SWLibrary.progressMeter);
};

XULSword.prototype.downloadCancelled = function(theWindow) {
	SWLibrary.download.finalize(true);
};

