/*  This file is part of xulSword.

    Copyright 2009 John Austin (gpl.programs.info@gmail.com)

    xulSword is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    xulSword is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with xulSword.  If not, see <http://www.gnu.org/licenses/>.
*/


// INITIALIZES GLOBAL COMMON VARS WHICH ONLY NEED TO BE EVALUATED ONCE

/************************************************************************
 * Initialize program-wide global variables
 ***********************************************************************/ 

XULSword.prototype.LocaleConfigs = {};


/************************************************************************
 * MODULE INIT ROUTINE
 ***********************************************************************/ 

XULSword.prototype.initModules = function() {
//	FIXME what's up here?
	//ModuleConfigDefault = getModuleConfig("LTR_DEFAULT");
  XULSword.prototype.ModuleConfigs = {}; //must set *prototype* not this.ModuleConfigs
  // Gets list of available modules
  var modules = LibSword.getModuleList();
  if (!modules || modules == "No Modules") return false;
  modules = modules.split("<nx>");
  
  for (var m=0; m<modules.length; m++) {
  
    var mod = modules[m].split(";")[0];
    var type = modules[m].split(";")[1];
        
    // Weed out unsupported module types
    var supported = false;
    for each (var stype in this.SupportedModuleTypes) {supported |= (type == stype);}
    if (!supported) {
      this.jsdump("ERROR: Dropping module \"" + mod + "\". Unsupported type \"" + type + "\".");
      continue;
    }
    
    // Weed out incompatible module versions. The module installer shouldn't 
    // allow bad mods, but this is just in case.
    var comparator = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
    
    var xsengvers = LibSword.getModuleInformation(mod, "MinimumVersion");
    xsengvers = (xsengvers!=this.NOTFOUND ? xsengvers : 0);
    var enginevers = "1.7.3";
    //var enginevers; try {enginevers = prefs.getCharPref("EngineVersion");} catch (er) {enginevers = this.NOTFOUND;}
    if (enginevers != this.NOTFOUND && comparator.compare(enginevers, xsengvers) < 0) {
      this.jsdump("ERROR: Dropping module \"" + mod + "\". enginevers:" + enginevers + " < " + "xsengvers:" + xsengvers);
      continue;
    }
    
    this.ModuleConfigs[mod] = this.getModuleConfig(mod);
    
    // if a font is specified in a conf, write it to user pref as well. This
    // allows an XSM module to specify a font for a module which will
    // persist even if the module is updated from another repo. Order of
    // cascade is: default-val -> conf-val -> user-pref-val
    var toUpdate = ["fontFamily", "fontSize", "lineHeight", "color", "background", "fontSizeAdjust"];
    for (var i=0; i<toUpdate.length; i++) {
			var confVal = LibSword.getModuleInformation(mod, this.Config[toUpdate[i]].modConf);
			if (confVal != this.NOTFOUND && !(/^\s*$/).test(confVal)) {
				// don't auto-overwrite if it's already set!
				try {
					var test = this.prefs.getCharPref("user." + toUpdate[i] + "." + mod);
				}
				catch (er) {this.prefs.setCharPref("user." + toUpdate[i] + "." + mod, confVal);}
			}
		}
		
  }
  
  return true;
}



/************************************************************************
 * RUN ALL THE INITIALIZATION ROUTINES
 ***********************************************************************/ 

XULSword.prototype.xulswordInit = function() {

  var defaultMod = this.NOTFOUND;
 
  this.initModules();
  
  this.prefs.setCharPref("DefaultVersion", defaultMod);

  if (!LibSword.loadFailed) LibSword.unlock();
  
}
