/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008-2012 Brian Fernandes <brian@thegoan.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

var EXPORTED_SYMBOLS = ["BibleChannel"];

function BibleChannel(uri, jsword, wm) {
	this.URI = uri;
	this.originalURI = uri;
	this._isPending = false;
	this.contentCharset = "UTF-8";
	this.jsword = jsword;
	this.windowManager = wm;
};


BibleChannel.prototype = {
	loadAttributes: null,
	contentType: "text/html",
	contentLength: -1,
	owner: null,
	loadGroup: null,
	notificationCallbacks: null,
	securityInfo: null,
	status: Components.results.NS_OK,
	annotSvc: Components.classes["@mozilla.org/browser/annotation-service;1"].
				getService(Components.interfaces.nsIAnnotationService),
	taggingSvc: Components.classes["@mozilla.org/browser/tagging-service;1"].
    			getService(Components.interfaces.nsITaggingService),


	cancel: function(status) {
		this.jsword.log("cancel");
	},

	isPending: function() {
		this.jsword.log("pending");
		return this._isPending;
	},

	suspend: function() {
		this.jsword.log("suspend " + this.notificationCallbacks);
		//this.loadGroup.removeRequest(this, null, Components.results.NS_OK);
		throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
	},

	resume: function() {
		this.jsword.log("resume");
		throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
	},

	QueryInterface: function (iId) {
		if (!iId.equals(Components.interfaces.nsIChannel) && !iId.equals(Components.interfaces.nsIRequest)
			&& !iId.equals(Components.interfaces.nsISupports)) {
			throw Components.results.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	asyncOpen: function(observer, context) {
		this._isPending = true;
		try {
			if (this.loadGroup) {
				this.loadGroup.addRequest(this, null);
			}
			var htmlData = this.jsword.isInitialized() ? this.getHTML() : ["FireBible is not initialized", null];
			var html = htmlData[0];
			observer.onStartRequest(this, context);
			var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
							.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
			converter.charset = "UTF-8";
			var bst = converter.convertToInputStream(html);
			var len = bst.available();
			observer.onDataAvailable(this, context, bst, 0, len);
			//bst.close();
			observer.onStopRequest(this, context, Components.results.NS_OK);
			if (this.loadGroup) {
        		this.loadGroup.removeRequest (this, null, Components.results.NS_OK);
        	}
			this._isPending = false;
			if (htmlData[1]) {
				this.setAnnotation(htmlData[1]);
			}
		} catch (e) {
			this.jsword.log("error " + e);
		}
	},

	open: function() {
		throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
	},

	getHTML: function() {
		var win = this.windowManager.getMostRecentWindow("navigator:browser");
		return win.fireBible.load(this.URI);
	},

	setAnnotation: function(bookKeyString) {
		var module = this.jsword.getModuleArray(bookKeyString[0][0]);
		if (!module) return;
		var bClass = module.bClass;
		var bCat = module.bCat;
		this.annotSvc.setPageAnnotation(this.URI, "firebible/bookCategory/" + bCat,
																		bClass, 0,
																		this.annotSvc.EXPIRE_WITH_HISTORY);
		this.taggingSvc.tagURI(this.URI, ["firebible", bClass]);
		var win = this.windowManager.getMostRecentWindow("navigator:browser");
		if (bClass == "bible" || bClass == "questionable") {
			//load commentary
			win.fireBible.lastBibleURI = bookKeyString[1];
			var sidebarWindow = win.document.getElementById("sidebar").contentWindow;
			if (sidebarWindow.location.href == "chrome://firebible/content/sidebar.xul") {
				sidebarWindow.fbItemList.loadCommentary();
			}
		}
	}
};
